/* DNA-View - dna.cpp
Here you find all declarations of methods concerning DNA dingle and double strings */

/**
Copyright (C) 2003, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/



#include "dna.h"

#define FULL 100		// max number of chars to read "at once" from DNA file

/*************************************************************************************/

static const QRegExp rx("[ACGT]");		// regular expression for search

static char cComplement(char z) {			// returns the complement of a base A, C, G, T
	char w;
	switch (z) {
 	 case 'A': w='T'; break;
 	 case 'C': w='G'; break;
 	 case 'G': w='C'; break;
 	 case 'T': w='A'; break;
 	 default :	QMessageBox messageBox;
			messageBox.critical(0,
				version + " - Error in cComplement",
				"There was an error in function cComplement\nbecause a character different from 'ACGT' was found.\n\nThis should not happen! Please inform me.",
				"&Ok", QString::null, QString::null, 0, 0);
		     	return(0);
			break;
	}
	return (w);
}

/****************************************************************/
/******* declarations for class dnastring *******/

dnastring::dnastring(char *str) : QCString(str) {}		// standard constructor

dnastring::dnastring(dnastring& ds) : QCString(ds) {		// copy constructor
	setStr(ds.data());
}

uint dnastring::rlength(void) const {	// returns lenght of a string as numbers of bases
	if(isNull()) {
		     QMessageBox messageBox;
		     messageBox.critical(0,
			version + " - Error in dnastring::rlength",
			"There was an error in function rlength\nbecause of an Null-String.\n\nThis should not happen! Please inform me.",
			"&Ok", QString::null, QString::null, 0, 0);
	}
	return(length() - contains(' '));	// real length minus number of spaces
}

dnastring dnastring::sComplement(void) const {	// returns the base complement of a string
	uint i, len=length();
	char *strc;
	dnastring ds;

	strc=new char[len+1];		// allocate memory for the new complement string
	Q_CHECK_PTR(strc);
	char *go1=strc, *go2=data();

	for (i=1;i<=len;i++) {
		*go1=cComplement(*go2);		// *go1 is the char complement of *go2
		if (!go1) {
		     QMessageBox messageBox;
		     messageBox.critical(0,
			version + " - Error in dnastring::sComplement",
			"There was an error in function sComplement\nbecause a character different from 'ACGT' was found.\n\nThis should not happen! Please inform me.",
			"&Ok", QString::null, QString::null, 0, 0);
		     return(ds);
		}
		go1++; go2++;
	}
	*go1='\0';

	ds.setStr(strc);		// copies the complement in a DNA single string
	delete (strc);		// frees the allocated memory
	return (ds);
}

void dnastring::toupper(void) {		// converts a string to upper case letters
	setStr((upper()).data());
}

void dnastring::blunt(void) {			// cuts the hanging end of a string

	uint end=findRev(rx,length());		// find first non-space from behind
	end++;
	truncate(end);					// truncate the string at the last base
}

uint dnastring::isnotCorrect(void) const {		// checks if a string is not "correct" and returns 0 or first pos where it is

	QCString str=stripWhiteSpace();		// deletes spaces at the beginning and end
	uint pos=0;

	if (str.isEmpty()) return(pos);		// string is only made of spaces, it's ok
	if (str.find(' ')<0) return(pos);		// string is okay - there's no space left

	pos=str.find(' ')+1;	// finds first space
	pos+=find(rx);		// adding beginning spaces to position

	return(pos);
}

int dnastring::cut(uint start, uint end) {		// cuts a single string from start to end

	uint i;
	if (start>end) { i=start; start=end; end=i; }		// switch start and end

	if (start<=0 || start>length()) return (1);	// start is to far away
	if (end<=0 || end>length()) return (1);	// end is to far away

	truncate(end);		// cuts the end

	char *go=data()+start-1;
	// setStr(go);			// old cut, crashed sometimes
	qstrcpy(data(),go);		// "cuts" the start
	return (0);
}

void dnastring::fill(char c, uint start, uint end) {			// fills a string with character c from start to end

	if (start<=0 || start>length()) return;	// start is to far away
	if (end<=0 || end>length()) return;		// end is to far away

	uint i;
	if (start>end) { i=start; start=end; end=i; }		// switch start and end

	char *string=data();
	char *go=string+start-1;
	for (i=0;i<=end-start;i++) {		// fills string with characters
		*go=c;
		go++;
	}
	setStr(string);
}

bool dnastring::isEmpty(void) {					// returns TRUE if string is "empty" (empty also includes only spaces in a string)
	if ((stripWhiteSpace()).length()) return (FALSE);
	else return (TRUE);
}

int dnastring::del(uint pos) {				// deletes character at position pos

	if (pos<1 || pos>length()) return(1);	// pos is too far away

	dnastring temp1(data());
	temp1.truncate(pos-1);		// gets the beginning
	char *temp2=data()+pos;	// gets the end

	setStr(temp1+temp2);		// concatenate strings without character at position pos
	return(0);
}

char dnastring::getc(uint pos) {		// returns character at position pos or NULL on error

	if (pos<1 || pos>length()) return('\0');		// pos is too far away
	char *go=data()+pos-1;				// sets *go at the right position
	return(*go);
}


/******************************************************************************************************************************/
/******* declarations for class dnadstring *******/

dnadstring::dnadstring(void) {			// standard constructor
	set("","");
}

dnadstring::dnadstring(dnadstring& dds) {	// copy constructor
	set(dds.getf(),dds.gett());
}

bool dnadstring::operator==(dnadstring d) const {		// compares two DNA double strings
	return (fstring==d.fstring && tstring==d.tstring);
}

bool dnadstring::operator!=(dnadstring d) const {		// compares two DNA double strings
	return(d.fstring!=fstring || d.tstring!=tstring);
}

int dnadstring::operator+=(dnadstring d) {		// concatenate two DNA double strings
	int mode=4*isEmpty()+d.isEmpty();	// calculates number for each special  situation of concatenation

	/*
		0 means		dnastring is full and d.dnastring is full
		1 means		dnastring is full and d.dnastring is empty in fstring
		2 means		dnastring is full and d.dnastring is empty in tstring
		3 means		dnastring is full and d.dnastring is empty
		4 means		dnastring is empty in fstring and d.dnastring is full
		5 means		dnastring is empty in fstring and d.dnastring is empty in fstring
		6 means		dnastring is empty in fstring and d.dnastring is empty in tstring
		7 means		dnastring is empty in fstring and d.dnastring is empty
		8 means		dnastring is empty in tstring and d.dnastring is full
		9 means		dnastring is empty in tstring and d.dnastring is empty in fstring
		10 means		dnastring is empty in tstring and d.dnastring is empty in tstring
		11 means		dnastring is empty in fstring and d.dnastring is empty
		12 means		dnastring is empty and d.dnastring is full
		13 means		dnastring is empty and d.dnastring is empty in fstring
		14 means		dnastring is empty and d.dnastring is empty in tstring
		15 means		dnastring is empty and d.dnastring is empty
	*/

	dnadstring ds;
	ds.set(getf(), gett());
	dnastring ftemp1, ftemp2, ttemp1, ttemp2;

	QMessageBox messageBox;
	int hf;	// hanging front from the DNA that is to paste
	int res;	// result if there are two many possibilities
	uint i;

	int *posFound=(int*)malloc(2*sizeof(int));		// array for different positions where it could paste to
	Q_CHECK_PTR(posFound);

	// printf("mode: %ld\n",mode);

	switch (mode) {
	     case 6:
	     case 9: {
				dnastring fgo,		// string to search in
						sgo;		// search string
				uint len, flen=0;
				uint number=0, ft=0;
				int found=0;

			     	if (mode==9) {
	     				fgo=ds.getf();
	     				ftemp1=d.gett();
				}
				else {		// mode==6
	     				fgo=ds.gett();
	     				ftemp1=d.getf();
				}
				fgo=(fgo.stripWhiteSpace()).data();
				ftemp1=(ftemp1.stripWhiteSpace()).data();

				if (fgo.length()>=ftemp1.length()) sgo=ftemp1.sComplement();
				else {			// sgo should always be smaller than fgo
					sgo=fgo;
					fgo=ftemp1.sComplement();
				}

				len=fgo.length();
				number=fgo.contains(sgo);		// counts number of sgo in fgo

				// printf("number: %ld\n",number);

				switch (number) {
				   case 1: found=fgo.find(sgo)+1;		// found only one time
				   		break;
				   case 0: 	ftemp1=sgo;		// didn't found => search for part of string
						do {
							found=fgo.findRev(ftemp1)+1;				// search backwards
							if (len-found+1>ftemp1.length()) found=0;	// found in middle of string
							ftemp1=(ftemp1.left(ftemp1.length()-1)).data();	// cuts one letter away
						} while (!found && !ftemp1.isEmpty());
						if (found) {	// position for pasting was found
							ft=found;		// stores position
							flen=ftemp1.length()+1;
							number++;
							posFound[number-1]=found;	// stores position in array
						}
						ftemp1=sgo;
						do {
							found=fgo.find(ftemp1)+1;	// search forward
							if (found>1) found=0;		// found in middle of string
							if (found) {
								if (flen<ftemp1.length()) {
									if (flen) {		// second position found for pasting
										number++;
										posFound[number-1]=-1*sgo.length();	// stores position in array
										found=posFound[number-1];
									}
									flen=ftemp1.length();
								}
								else if (flen==ftemp1.length()) {
									number++;
									posFound[number-1]=-1*sgo.length();	// stores position in array
									found=posFound[number-1];
								}
								else {
									found=ft;
								}
							}
							ftemp1=(ftemp1.right(ftemp1.length()-1)).data();	// cuts one letter away
						} while (!found && !ftemp1.isEmpty() && ftemp1.length()>=flen);
						if (!found) found=ft;		// remembers last found position from backward search
						break;
				   default:	posFound=(int*)realloc(posFound,number*sizeof(int));
				   							// allocated memory for different positions
						Q_CHECK_PTR(posFound);
						posFound[0]=fgo.find(sgo)+1;		// found many positions for pasting
						for (i=1;i<number;i++) {
							posFound[i]=fgo.find(sgo,posFound[i-1])+1;		// stores all positions
						}
					break;
				}

				// printf("number: %ld\n",number);
				// printf("found: %ld\n",found);

				switch (number) {
				  case 0:	return(1); break;		// could not paste because doesn't match anywhere
				  case 1:
						if (found>0) {
				  			ftemp1=ds.getf();
	     						ttemp1=ds.gett();
							ftemp2=d.getf();
							ttemp2=d.gett();
						}
						else {
				  			ftemp1=d.getf();
	     						ttemp1=d.gett();
							ftemp2=ds.getf();
							ttemp2=ds.gett();
							found*=-1;
							if (mode==9) mode=6;		// changes paste origin and target
							else mode=9;
						}
						if (mode==9) {
							if (ftemp1.length()>=ttemp2.length()) {
								ttemp1.insert(found-1,ttemp2);
								ds.set(ftemp1,ttemp1);
							}
							else {
								ftemp2.insert(found-1,ftemp1);
								ds.set(ftemp2,ttemp2);
							}
						}
						else {
							if (ttemp1.length()>=ftemp2.length()) {
								ftemp1.insert(found-1,ftemp2);
								ds.set(ftemp1,ttemp1);
							}
							else {
								ttemp2.insert(found-1,ttemp1);
								ds.set(ftemp2,ttemp2);
							}
						}
						break;
				  default: 			// too many possibilities
						res=messageBox.warning(0,
						version + " - Concatenate DNA strings",
						"There are too many possibilities to concatenate these two strings.\nWould you like to paste it at a random position", "&Ok", "&Cancel", QString::null, 0, 1);
						if (res==0) {
							uint randomPos=(rand() % number);	// random number between 0 and number-1
							found=posFound[randomPos];

							// printf("%ld %ld %ld\n",number,randomPos,found);

							if (found>0) {
					  			ftemp1=ds.getf();
	     							ttemp1=ds.gett();
								ftemp2=d.getf();
								ttemp2=d.gett();
							}
							else {
					  			ftemp1=d.getf();
	     							ttemp1=d.gett();
								ftemp2=ds.getf();
								ttemp2=ds.gett();
								found*=-1;
								if (mode==9) mode=6;		// changes paste origin and target
								else mode=9;
							}
							if (mode==9) {
								if (ftemp1.length()>=ttemp2.length()) {
									ttemp1.insert(found-1,ttemp2);
									ds.set(ftemp1,ttemp1);
								}
								else {
									ftemp2.insert(found-1,ftemp1);
									ds.set(ftemp2,ttemp2);
								}
							}
							else {
								if (ttemp1.length()>=ftemp2.length()) {
									ftemp1.insert(found-1,ftemp2);
									ds.set(ftemp1,ttemp1);
								}
								else {
									ttemp2.insert(found-1,ttemp1);
									ds.set(ftemp2,ttemp2);
								}
							}
						}
						break;
				}
				break;
			  }
	     case 4:
	     case 8:	ds.fit();
				ftemp1=ds.getf();
	     			ttemp1=ds.gett();
				d.fit();
				ftemp2=d.getf();
	     			ttemp2=d.gett();
				hf=d.hangf();
				if ((mode==4 && hf<=0) || (mode==8 && hf>=0)) {
					ftemp1+=ftemp2;
					ttemp1+=ttemp2;
				}
				else {
					if (mode==4) {
						ttemp1+=(ttemp2.stripWhiteSpace()).data();
						ftemp1.insert(ds.length()-hf,ftemp2.data());
					}
					else {	// mode==8
						ftemp1+=(ftemp2.stripWhiteSpace()).data();
						ttemp1.insert(ds.length()+hf,ttemp2.data());
					}
				}
				ds.set(ftemp1,ttemp1);
				break;
	     case 0:
	     case 1:
	     case 2:	if (mode==0 && ds.hangb()+d.hangf()) return(1);
	     			ftemp1=ds.getf();
	     			ttemp1=ds.gett();
				ftemp1.blunt();
				ttemp1.blunt();
				ftemp2=d.getf();
	     			ttemp2=d.gett();
				ftemp1+=(ftemp2.stripWhiteSpace()).data();
				ttemp1+=(ttemp2.stripWhiteSpace()).data();
				ds.set(ftemp1,ttemp1);
				break;
	     case 5:
	     case 10:	ftemp1=getf();
	     			ftemp1+=d.getf();
				ttemp1=gett();
				ttemp1+=d.gett();
	     			ds.set(ftemp1, ttemp1);
	     			break;
	     case 12:
	     case 13:
	     case 14: 	ds.set(d.getf(), d.gett());
				break;
	     case 3:
	     case 7:
	     case 11:
	     case 15:
	     default: 	return(3); break;	// nothing to paste
	}

	// printf("|%s|\n|%s|\n",ds.getf(),ds.gett());

	if (ds.isnotCorrect()) return (1);
	set(ds.getf(), ds.gett());
	fit();
	return(0);
}

void dnadstring::setf(char *str) {			// sets the 5'-3' string
	fstring.setStr(str);
}

void dnadstring::sett(char *str) {			// sets the 3'-5' string
	tstring.setStr(str);
}

void dnadstring::setf(dnastring str) {		// sets the 5'-3' string
	fstring=str;
}

void dnadstring::sett(dnastring str) {		// sets the 3'-5' string
	tstring=str;
}

void dnadstring::set(char *str1, char *str2) {			// sets the DNA string
	setf(str1);
	sett(str2);
}

void dnadstring::set(dnastring str1, dnastring str2) {		// sets the DNA string
	setf(str1);
	sett(str2);
}

uint dnadstring::length(void) const {		// returns length of the DNA string
	return (MAX(lengthf(),lengtht()));
}

uint dnadstring::lengthf(void) const {		// returns length of the 5'-3' string
	return (fstring.length());
}

uint dnadstring::lengtht(void) const {		// returns length of the 3'-5' string
	return (tstring.length());
}

uint dnadstring::rlengthf(void) const {		// returns length of the 5'-3' string as bases
	return (fstring.rlength());
}

uint dnadstring::rlengtht(void) const {		// returns length of the 3'-5' string as bases
	return (tstring.rlength());
}

uint dnadstring::rlength(void) {			// returns length of DNA string as pair of bases
	uint hf=abs(hangf()), hb=abs(hangb());
	return (length()-hf-hb);
}

char *dnadstring::getf(void) const {			// returns the 5'-3' string
	return(fstring.data());
}

char *dnadstring::gett(void) const {			// returns the 3'-5' string
	return(tstring.data());
}

int dnadstring::hangf(void) {		// returns positive or negative numbers of hanging bases at the beginning

	int len=MAX(fstring.find(rx),tstring.find(rx));	// ignores empty spaces at the beginning
	int mult=1;

	if (len<0) len=0;				// DNA string is empty
	if (*(getf())==' ') mult=-1;		// negative value if tstring is longer at the beginning
	return(mult*len);
}

int dnadstring::hangb(void) { 	// returns positive or negative numbers of hanging bases at the end

	int len=length()-MIN(fstring.findRev(rx,lengthf()),tstring.findRev(rx,lengtht()))-1;	// ignores empty spaces at the end
	int mult=1;

	if (len<0) len=0;					// DNA string is empty
	if (*(getf()+lengthf()-1)==' ') mult=-1;	// negative value if tstring is longer at the end
	return(mult*len);
}

dnastring dnadstring::fCompl(void) const {	// calculates the base complement of the 5'-3' string
	dnastring temp;
	temp=fstring.sComplement();
	return (temp);
}

dnastring dnadstring::tCompl(void) const {	// calculates the base complement of the 3'-5' string
	dnastring temp;
	temp=tstring.sComplement();
	return(temp);
}

uint dnadstring::count (char c1, char c2) {	;// counts the bases in the double string, if c2 is empty it counts c1 in the single strings
	if (c2=='\0') {
		return(fstring.contains(c1,FALSE)+tstring.contains(c1,FALSE));		// returns number of c1 in both single strings
	}
	if (c1!=cComplement(c2)) return(0);				// c1 and c2 are not a matching base pair
	QCString temp=fstring.mid(abs(hangf()),rlength());		// copies out only the base pairs in fstring
	return(temp.contains(c1,FALSE));					// returns number of c1 in cpoied fstring
}

void dnadstring::upper(void) {	// converts the DNA string after read to upper case letters
	fstring.toupper();
	tstring.toupper();
}

uint dnadstring::isnotCorrect(void) {		// checks if a dna string is not "correct" and returns 0 or first pos where it is
	uint pos=0, i,j;

	i=fstring.isnotCorrect();
	j=tstring.isnotCorrect();
	if (i!=0 && j!=0) return(MIN(i,j));		// at least one string is not correct
	if (MAX(i,j)!=0) return(MAX(i,j));

	uint k=isEmpty();

	switch (k) {
	   case NONE_EMPTY: {		// both single are not empty
				uint hf=abs(hangf()), hb=abs(hangb());
				if (hf+hb>=length()) return(hf+1);			// there is no base pair in the string which is not correct

				dnastring fstr(fstring), tstr(tstring);
				uint flen=fstr.length(), tlen=tstr.length();

				fstr.cut(hf+1,MAX(flen,tlen)-hb);			// cuts out only double bases
				tstr.cut(hf+1,MAX(flen,tlen)-hb);

				if (fstr!=tstr.sComplement()) {				// at one position the single strings do not match
					char *go1=fstr.data(), *go2=tstr.data();

					pos+=hf+1;		// adding hanging front to position

					while (*go1!='\0') {
						if (*go1!=cComplement(*go2)) break;		// bases do not match here
						pos++; go1++; go2++;
					}
				}
		      		break;
			}
	   case FIVE_EMPTY:			// empty single strings or a whole empty string is correct
	   case THREE_EMPTY:
	   case BOTH_EMPTY:
	   default: break;
	}
	return(pos);
}

int dnadstring::isEmpty(void) {
	int result=NONE_EMPTY;					// both single strings are not empty
	if (fstring.isEmpty()) result+=FIVE_EMPTY;		// the 5'-3' string is empty
	if (tstring.isEmpty()) result+=THREE_EMPTY;		// 'the 3'-5' string is empty
	return(result);							// per addition the DNA strings is empty if (result==FIVE_EMPTY+THREE_EMPTY)
}

int dnadstring::read(QString file) {			// reads a DNA string from a file
	QFile data(file);

 	if (!data.open(IO_ReadOnly)) {
		QMessageBox messageBox;
		messageBox.information(0,
			version + " - Error in DNA file",
			"Could not open file "+file+".",
			"&Ok", QString::null, QString::null, 0, 0);
		return(1);
	}

	int chs, ch;		// chars in data
	uint line=0;		// count lines in data
	int i=0;

	char *fs, *ts, *fgo, *tgo;			// strings to save characters from file
	uint tnum=1, fnum=1, tlen=0, flen=0;

	fs=(char*)malloc((FULL+1)*sizeof(char));
	Q_CHECK_PTR(fs);

	ts=(char*)malloc((FULL+1)*sizeof(char));
	Q_CHECK_PTR(ts);

	fgo=fs; tgo=ts;
	*fgo='\0'; *tgo='\0';		// clear both strings

	do {
		chs=data.getch();		// reads character
		line++;				// counts up the lines in file
      		if (chs=='3' || chs=='5') {	// line is beginning with a 3 or a 5 as it shoud be
         		do {
			   ch=data.getch();	// reads character
            		   switch (ch) {
            		    case ' ':
            		    case 'A':
            		    case 'a':
            		    case 'C':
            		    case 'c':
            		    case 'G':
            		    case 'g':
            		    case 'T':
			    case 't':	if (chs=='3') {				// starts reading the 3-5 string

						if (tlen==FULL*tnum) {		// string is full and new memory must be allocated
							tnum++;
							ts=(char*)realloc(ts,FULL*tnum+1);
							Q_CHECK_PTR(ts);
							tgo=ts+tlen-1;		// sets the new start at the end of all chars read yet
						}
						if (tlen) tgo++;
                       				*tgo=ch;
						*(tgo+1)='\0';
						tlen++;
					}
					else {					// starts reading the 5-3 string

						if (flen==FULL*fnum) {		// string is full and new memory must be allocated
			    				fnum++;
							fs=(char*)realloc(fs,FULL*fnum+1);
							Q_CHECK_PTR(fs);
							fgo=fs+flen-1;		// sets the new start at the end of all chars read yet
						}
						if (flen) fgo++;
                       				*fgo=ch;
						*(fgo+1)='\0';
						flen++;
					}
					break;
            		    case '\n': 	break;
            		    default:	QMessageBox messageBox;				// wrong character was read
						QString temp;
						temp=temp.setNum(line);
						messageBox.information(0,version + " - Error in DNA file", "There was an error in file "+file+"\nin line "+temp+" while reading.", "&Ok", QString::null, QString::null, 0, 0);
						data.close();
						return(1);
            		   }
         	   	} while (ch!='\n' && ch!=EOF);		// read till end of line or end of file
     		}
      		else if (chs!=EOF) {		// line is not beginning with a 3 or a 5 and it's not the end of file
			QMessageBox messageBox;
			QString temp;
			temp=temp.setNum(line);
			messageBox.information(0, version + " - Error in DNA file",
				"There was an error in file "+file+"\nin line "+temp+" while reading.",
				"&Ok", QString::null, QString::null, 0, 0);
			data.close();
			return(1);
      		}
	} while (chs!=EOF);
	data.close();

	set(fs,ts);
	upper();			// converts the characters to upper case letters
	fit();				// fits the DNA string in a correct format
	i=isnotCorrect();		// checks again if DNA string is correct or not
	if (i) {
		QMessageBox messageBox;
		messageBox.information(0, version + " - Error in DNA file",
			"There was an error in file "+file+"\nwhile reading.",
			"&Ok", QString::null, QString::null, 0, 0);
		set("","");		// if DNA string was not correct it is set to an empty DNA string
	}
	return(i);
}

int dnadstring::write(const QString file) const {		// writes a DNA string to a file
	QFile data(file);

 	if (!data.open(IO_WriteOnly)) {
		QMessageBox messageBox;
		messageBox.information(0,
			version + " - Error in DNA file",
			"Could not open file "+file+".",
			"&Ok", QString::null, QString::null, 0, 0);
		return(-1);
	}

	const char *fgo=fstring, *tgo=tstring;
	uint i;

	/* the following passage writes the 5'-3' and 3'-5' string with length FULL alternating in the specific file */

	while (*fgo!='\0' || *tgo!='\0') {
		if (*fgo!='\0') {
			i=0;
			data.putch('5');
			while (*fgo!='\0' && i<FULL) {
				data.putch(*fgo);
				fgo++;
				i++;
			}
			data.putch('\n');
		}
		if (*tgo!='\0') {
			i=0;
			data.putch('3');
			while (*tgo!='\0' && i<FULL) {
				data.putch(*tgo);
				tgo++;
				i++;
			}
			data.putch('\n');
		}
	}
	data.close();
	return(0);
}

void dnadstring::fit(void) {		// fills or delete spaces so that lengthf=lengtht
	fstring.blunt();
	tstring.blunt();

	uint lenf=lengthf(), lent=lengtht();

	if (lenf<lent) fstring.insert(lent-1,' ');			// insert spaces at the end if necessary
	else if (lenf>lent) tstring.insert(lenf-1,' ');

	// cuts front hanging double spaces
	uint m=fstring.find(rx)+1;
	if (m==0) {				// fstring is empty
		m=tstring.find(rx)+1;
		if (m==0) set("","");			// both single strings are empty
	}
	else {
		uint n=tstring.find(rx)+1;
		if (n<m && n>0) m=n;
	}
	fstring.cut(m,lengthf());
	tstring.cut(m,lengtht());
}

void dnadstring::blunt(void) {		// completes a dna string and cuts the hanging end
	fstring.blunt();
	tstring.blunt();
	uint min=MIN(lengtht(),lengthf());

	if (min==0) {			// one of the two strings is empty
		if (fstring.isEmpty()) fstring=tCompl();			// 5'-3' string is empty
		else if (tstring.isEmpty()) tstring=fCompl();		// 3'-5' string is empty
	}
	else {			// both strings are pot empty
		fstring.truncate(min);		// cuts the hanging edge
		tstring.truncate(min);
		if (hangf()>0) tstring=fCompl();			// completes the DNA string with it's bigger complement
		else if (hangf()<0) fstring=tCompl();
	}
}

int dnadstring::cut(uint fstart, uint fend, uint tstart, uint tend) {
	// cuts the dnastring in fstring from fstart to fend and in tstring from tstart to tend

	if (!tend || !tstart) {
		tend=0; tstart=0;
	}

	if (!fend || !fstart) {
		fend=0; fstart=0;
	}

	if (!fstart && !tstart) return(1);		// nothing to cut

	uint i;
	if (fstart>fend) { i=fstart; fstart=fend; fend=i; }		// switch fstart and fend
	if (tstart>tend) { i=tstart; tstart=tend; tend=i; }		// switch tstart and tend

	dnastring fstring(getf()), tstring(gett());

	if (!fstart) {						// copying only 3'-5' string (tstring)
		if (tstart>lengtht()) return (1);	// tstart is too far away
		if (tend>lengtht()) return (1);		// tend is too far away

		tstring.cut(tstart,tend);			// tstring is cut...
		fstring="";					// ... and fstring is empty
	}
	else {
		if (fstart>lengthf()) return (1);	// fstart is too far away
		if (fend>lengthf()) return (1);		// fend is too far away

		if (!tstart) {					// copying only 5'-3' string (fstring)
			fstring.cut(fstart,fend);		// fstring is cut ...
			tstring="";				// ... and tstring is empty
		}
		else {						// copying complete string
			if (tstart>lengtht()) return (1);	// tstart is too far away
			if (tend>lengtht()) return (1);		// tend is too far away

			if ((fstart || tstart) && (fstart>tend || tstart>fend)) return (1);		// cut would lead to an incorrect string

			uint start=MIN(fstart,tstart), end=MAX(fend,tend);			// absolute start and end
			fstring.cut(start,end);								// cuts a dna block out
			tstring.cut(start,end);

			fstring.fill(' ',1,fstart-start);				// fills fstring with necessary spaces
			fstring.fill(' ',fend-start+2,fstring.length());
			tstring.fill(' ',1,tstart-start);				// fills tstring with necessary spaces
			tstring.fill(' ',tend-start+2,tstring.length());
		}
	}

	dnadstring ds;			// creates an temporary DNA string to check if everything is fine
	ds.set(fstring,tstring);
	ds.fit();
	if (ds.isnotCorrect()) return(1);
	else set(ds.getf(),ds.gett());
	return(0);
}

dnadstring dnadstring::copy(uint fstart, uint fend, uint tstart, uint tend) {
	// copies the dnastring in fstring from fstart to fend and in tstring from tstart to tend

	dnadstring ds;
	ds.set("","");

	if (!tend || !tstart) {
		tend=0; tstart=0;
	}

	if (!fend || !fstart) {
		fend=0; fstart=0;
	}

	if (!fstart && !tstart) return(ds);		// nothing to copy

	uint i;
	if (fstart>fend) { i=fstart; fstart=fend; fend=i; }		// switch fstart and fend
	if (tstart>tend) { i=tstart; tstart=tend; tend=i; }		// switch tstart and tend

	dnastring fstring(getf()), tstring(gett());

	if (!fstart) {						// copying only 3'-5' string (tstring)
		if (tstart>lengtht()) return (ds);	// tstart is too far away
		if (tend>lengtht()) return (ds);		// tend is too far away

		tstring.cut(tstart,tend);			// tstring is cut...
		fstring="";					// ... and fstring is empty
	}
	else {
		if (fstart>lengthf()) return (ds);	// fstart is too far away
		if (fend>lengthf()) return (ds);		// fend is too far away

		if (!tstart) {					// copying only 5'-3' string (fstring)
			fstring.cut(fstart,fend);		// fstring is cut ...
			tstring="";				// ... and tstring is empty
		}
		else {						// copying complete string
			if (tstart>lengtht()) return (ds);	// tstart is too far away
			if (tend>lengtht()) return (ds);		// tend is too far away

			if ((fstart || tstart) && (fstart>tend || tstart>fend)) return (ds);		// cut would lead to an incorrect string

			uint start=MIN(fstart,tstart), end=MAX(fend,tend);			// absolute start and end
			fstring.cut(start,end);								// cuts a dna block out
			tstring.cut(start,end);

			fstring.fill(' ',1,fstart-start);				// fills fstring with necessary spaces
			fstring.fill(' ',fend-start+2,fstring.length());
			tstring.fill(' ',1,tstart-start);				// fills tstring with necessary spaces
			tstring.fill(' ',tend-start+2,tstring.length());
		}
	}

	ds.set(fstring,tstring);
	ds.fit();
	if (ds.isnotCorrect()) ds.set("","");

	return(ds);
}

int dnadstring::insert(uint pos, char fc, char tc, bool ins) {
	// inserts at position pos the char fc in fstring and tc in tstring, if ins is FALSE the character at pos will be overwritten

	if (pos<1 || pos>length()+1) return (2);		// pos is too far away
	dnastring fstr(getf()), tstr(gett());
	dnadstring temp;
	fstr.insert(pos-1, fc);
	tstr.insert(pos-1,tc);

	if (!ins) {				// if ins is FALSE the "old" character will be deleted
		fstr.del(pos+1);
		tstr.del(pos+1);
	}

	temp.set(fstr,tstr);
	if (temp.isnotCorrect()) return(1);		// checks again if the DNA string is correct
	else set(fstr,tstr);
	return(0);
}

int dnadstring::del(uint pos) {				// deletes base pair at position pos
	dnastring fstr(getf()), tstr(gett());
	int j=fstr.del(pos);		// deletes base at position pos
	int k=tstr.del(pos);
	if (j || k) return(1);	// there was an error while deleting

	dnadstring temp;
	temp.set(fstr,tstr);
	if (temp.isnotCorrect()) return(1);		// checks again if the DNA string is correct
	else set(fstr,tstr);
	return(0);
}

uint dnadstring::find(char *str, uint start, bool five) {
     						// returns first occurence of str from position start in fstring if five is TRUE or in tstring if five is FALSE
	uint pos=0;
	if (start<=0 || start>MIN(lengthf(),lengtht())) return (pos);	// start is too far away

	dnastring fgo1, fgo2;

	fgo2.setStr(str);
	fgo2.setStr(fgo2.stripWhiteSpace());		// ignores spaces at beginning and end
	if (five) fgo1.setStr(getf());
	else fgo1.setStr(gett());

	pos=fgo1.find(fgo2.data(),start-1)+1;		// finds first occurence of str
	return(pos);
}

uint dnadstring::find(dnastring str, uint start, bool five) {
 				// returns first occurence of str from position start in fstring if five is TRUE or in tstring if five is FALSE
	uint pos=0;
	if (start<=0 || start>MIN(lengthf(),lengtht())) return (pos);	// start is too far away

	dnastring fgo1, fgo2;

	fgo2.setStr(str.stripWhiteSpace());			// ignores spaces at beginning and end
	if (five) fgo1.setStr(getf());
	else fgo1.setStr(gett());

	pos=fgo1.find(fgo2.data(),start-1)+1;		// finds first occurence of str
	return(pos);
}

uint dnadstring::find(dnadstring ds, uint start) {		// returns first occurence of ds in the DNA string from position start
	uint pos=0;
	int hf=ds.hangf();

	if (start<=0 || start>MIN(lengthf(),lengtht())) return (pos);	// start is to far away

	dnastring fgo1,	// main search single string of d1
		  	sgo1,			// side search single string of d1
	  		fgo2,			// main search single string of d2
	  		sgo2;			// side search single string of d2
     						// I choose this to achieve a smaller code.
	dnastring temp1, temp2;

	int k=ds.isEmpty();		// checks if search DNA is kind of "empty"
	switch (k) {
	   case NONE_EMPTY:
				fgo2.setStr(ds.getf());
				sgo2.setStr(ds.gett());

				if (hf>=0) {			// 5'-3' string is main search argument
					fgo1.setStr(getf());
					sgo1.setStr(gett());
				}
				else {			// 3'-5' string is main search argument
					fgo1.setStr(gett());
					sgo1.setStr(getf());
				}

				fgo2.blunt();					// cuts the ending spaces
				sgo2.blunt();
				sgo2.cut(abs(hf)+1,sgo2.length());		// cuts the "important" part out

				do {
					pos=fgo1.find(fgo2.data(),start-1)+1;	// searches for main search argument
					if (pos) {						// main search argument exists in string
						temp1=sgo1;
						temp1.cut(abs(hf)+pos,abs(hf)+pos+sgo2.length()-1);
												// cuts the "important" part out
						if (temp1!=sgo2) start=pos+1;
						else break;				// dna string was found at pos
					}
				} while (pos);
		       	break;
	   case FIVE_EMPTY: pos=find(ds.gett(),start,FALSE); break;		// 5'-3' string in search DNA is empty
	   case THREE_EMPTY: pos=find(ds.getf(),start,TRUE); break;		// 3'-5' string in search DNA is empty
	   default: break;
	}
	return(pos);
}


