/* DNA-View - dna.h
Here you find all definitions of methods concerning DNA single and double strings */

/**
Copyright (C) 2003, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef DNA_H
#define DNA_H

#include <stdlib.h>		// needed for malloc and system

#include <qfile.h>
#include <qcstring.h>
#include <qstring.h>
#include <qregexp.h>
#include <qglobal.h>
#include <qmessagebox.h>

#define MAX(a,b) (a < b ? b : a)		// calculates the maximum of two numbers
#define MIN(a,b) (a < b ? a : b)		// calculates the minimum of two numbers

#define NONE_EMPTY 0			// definitions for different types of "empty" DNA double strings (see below)
#define FIVE_EMPTY 1
#define THREE_EMPTY 2
#define BOTH_EMPTY 3

extern const QString version;		// shows current version in top bar (defined in window.cpp)

class dnadstring;		// declaration of class dnadstring

class dnastring : public QCString {				// class for DNA single string

	friend class dnadstring;		// class dnadstring declared as friend s.t. it can call private methods of dnastring

	uint rlength(void) const;			// returns length of a string as number of bases
	dnastring sComplement(void) const;	// returns the base complement of a string or NUL on error

	void blunt(void);					// cuts the hanging end of a string
	int del(uint pos);					// deletes character at position pos
	void fill(char c, uint start, uint end); 	// fills a string with Character c from start to end
	void toupper(void);				// converts a string to upper case letters

	uint isnotCorrect(void) const;		// checks if a string is not "correct" and returns 0 or first pos where it is

    public:
	dnastring(char *str="");			// standard constructor
	dnastring(dnastring &ds);		// copy constructor

	int cut(uint start, uint end);			// cuts a single string from start to end (inclusive)
	bool isEmpty(void);				// returns TRUE if string is "empty"

	char getc(uint pos);		// returns character at position pos

};

class dnadstring {		// class for DNA double string

	dnastring fstring;		// 5'-3' string
	dnastring tstring;		// 3'-5' string

	void setf(char *str);			// sets the 5'-3' string
	void sett(char *str);			// sets the 3'-5' string
	void setf(dnastring str);			// sets the 5'-3' string
	void sett(dnastring str);			// sets the 3'-5' string

	uint lengthf(void) const;		// return length of the 5'-3' string
	uint lengtht(void) const;		// return length of the 3'-5' string

	dnastring fCompl(void) const;		// calculates the base complement of the 5'-3' string
	dnastring tCompl(void) const;		// calculates the base complement of the 3'-5' string

	void upper(void);					// converts the DNA string after read to upper case letters

	uint find(char *str, uint start, bool five);	     // returns first occurence of str from position start in fstring if five is TRUE or in tstring if five is FALSE
	uint find(dnastring str, uint start, bool five);  // returns first occurence of str from position start in fstring if five is TRUE or in tstring if five is FALSE

    public:
	dnadstring(void);					// standard constructor
	dnadstring(dnadstring& dds);		// copy constructor

	bool operator==(dnadstring d) const;		// compares two DNA double strings
	bool operator!=(dnadstring d) const;		// compares two DNA double strings
	int operator+=(dnadstring d);		// concatenate two DNA strings

	uint length(void) const;			// returns length of the string
	uint rlengthf(void) const;		// returns length of the 5-3 string as bases
	uint rlengtht(void) const;		// returns length of the 3-5 string as bases
	uint rlength(void);			// returns length of dna string as pair of bases

	void set(char *str1, char *str2);			// sets the dna string
	void set(dnastring str1, dnastring str2);		// sets the dna string

	char *getf(void) const;			// returns the 5'-3' string
	char *gett(void) const;			// returns the 3'-5' string

	int hangf(void);
		/* returns the number of hanging bases at the front (>0 for inter 5'-3' string,
		<0 for inter 3'-5' string and =0 for blunt) */

	int hangb(void);
		/* returns the number of hanging bases at the front (>0 for inter 5'-3' string,
		<0 for inter 3'-5' string and =0 for blunt) */

	uint count(char c1, char c2='\0');		// counts the bases in the double string, if c2 is empty it counts c1 in the single strings

	uint isnotCorrect(void);		// checks if a dna string is not "correct" and returns 0 or first pos where it is
	int isEmpty(void);			// returns different empty types (see above)

	int read(QString file);				// reads a dna string from a file
	int write(const QString file) const;		// writes a dna string to a file

	void fit(void);			// fills or delete spaces so that lengthf=lengtht
	void blunt(void);			// completes a dna strings and cuts the hanging end
	dnadstring copy(uint fstart, uint fend, uint tstart=0, uint tend=0);	// copies the dnastring in fstring from fstart to fend and in tstring from tstart to tend
	int cut(uint fstart, uint fend, uint tstart=0, uint tend=0);	// cuts the dnastring in fstring from fstart to fend and in tstring from tstart to tend
	int insert(uint pos, char fc, char tc, bool ins);			// inserts at position pos the char fc in fstring and tc in tstring
	int del(uint pos);								// deletes base pair at position pos

	uint find(dnadstring ds, uint start=1);		// returns first occurence of ds in the DNA string from position start

};

#endif
