/* DNA-View - opengl.cpp
Here you find all declarations of OpenGL procedure calls like paintGL */

/**
Copyright (C) 2003, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "opengl.h"

#define ADENIN 11		// defines the four bases for glCallList
#define CYTOSIN 12
#define GUANIN 13
#define THYMIN 14
#define EMPTYBASE 99
#define ACHAR 31		// defines the chars as graphic for the bases in 5'-3' string
#define CCHAR 32
#define GCHAR 33
#define TCHAR 34
#define AROTCHAR 41		// defines the chars as graphic for the bases in 3'-5' string (rotated)
#define CROTCHAR 42
#define GROTCHAR 43
#define TROTCHAR 44
#define REND 101		// defines the right yellow connection
#define LEND 102		// defines the left yellow connection

#define STANDARDCL blue
#define MARKEDCL cyan
#define CURRENTCL red

/******************************************************************************************************************************/

glWindow::glWindow( QWidget *parent, const char *name) : QGLWidget(parent, name) {	// standard constructor
	dist=0.65;		// calculated distance between two bases
	scale=1;		// scale factor set to 100%
}

void glWindow::setScale(GLfloat s) {		// sets the scale factor
	scale=s;
}

GLfloat glWindow::getScaleX(void) {		// returns the x scale factor relating to window width s.t. the bases has always the same width
	return(0.3*scale*600/width());
}

GLfloat glWindow::getScaleY(void) {		// returns the y scale factor relating to window height s.t. the bases has always the same height
	return(0.75*scale*250/height());
}

GLfloat glWindow::getDist(void) {			// returns the distance between two bases
	return(dist);
}

GLfloat glWindow::getScale(void) {		// returns the scale factor
	return(scale);
}

void glWindow::glTranslate(GLfloat x, GLfloat y, GLfloat z) {		// translates the scene
	glTranslatef(x,y,z);
}

void glWindow::glScale(GLfloat x, GLfloat y, GLfloat z) {		// scales the scene
	glScalef(x, y, z);
}

void glWindow::glRotateHalf(void) {			// rotates the scene 180 degree around the z-axis (needed for 3'-5' string)
	glRotatef(180.0,0.0,0.0,1.0);
}

void glWindow::addA(bool rot) {		// adds standard color Adenin to the scene
	qglColor(STANDARDCL);
	glCallList(ADENIN);
	if (rot) glCallList(AROTCHAR);
	else glCallList(ACHAR);
}

void glWindow::addC(bool rot) {		// adds standard color Cytosin to the scene
	qglColor(STANDARDCL);
	glCallList(CYTOSIN);
	if (rot) glCallList(CROTCHAR);
	else glCallList(CCHAR);
}

void glWindow::addG(bool rot) {		// adds standard color Guanin to the scene
	qglColor(STANDARDCL);
	glCallList(GUANIN);
	if (rot) glCallList(GROTCHAR);
	else glCallList(GCHAR);
}

void glWindow::addT(bool rot) {		// adds standard color Thymin to the scene
	qglColor(STANDARDCL);
	glCallList(THYMIN);
	if (rot) glCallList(TROTCHAR);
	else glCallList(TCHAR);
}

void glWindow::addMarkedA(bool rot) {		// adds marked color Adenin to the scene
	qglColor(MARKEDCL);
	glCallList(ADENIN);
	if (rot) glCallList(AROTCHAR);
	else glCallList(ACHAR);
}

void glWindow::addMarkedC(bool rot) {		// adds marked color Guanin to the scene
	qglColor(MARKEDCL);
	glCallList(CYTOSIN);
	if (rot) glCallList(CROTCHAR);
	else glCallList(CCHAR);
}

void glWindow::addMarkedG(bool rot) {		// adds marked color Cytosin to the scene
	qglColor(MARKEDCL);
	glCallList(GUANIN);
	if (rot) glCallList(GROTCHAR);
	else glCallList(GCHAR);
}

void glWindow::addMarkedT(bool rot) {		// adds marked color Thymin to the scene
	qglColor(MARKEDCL);
	glCallList(THYMIN);
	if (rot) glCallList(TROTCHAR);
	else glCallList(TCHAR);
}

void glWindow::addCurrentA(bool rot) {		// adds current color Adenin to the scene
	qglColor(CURRENTCL);
	glCallList(ADENIN);
	if (rot) glCallList(AROTCHAR);
	else glCallList(ACHAR);
}

void glWindow::addCurrentC(bool rot) {		// adds current color Cytosin to the scene
	qglColor(CURRENTCL);
	glCallList(CYTOSIN);
	if (rot) glCallList(CROTCHAR);
	else glCallList(CCHAR);
}

void glWindow::addCurrentG(bool rot) {		// adds current color Guanin to the scene
	qglColor(CURRENTCL);
	glCallList(GUANIN);
	if (rot) glCallList(GROTCHAR);
	else glCallList(GCHAR);
}

void glWindow::addCurrentT(bool rot) {		// adds current color Thymin to the scene
	qglColor(CURRENTCL);
	glCallList(THYMIN);
	if (rot) glCallList(TROTCHAR);
	else glCallList(TCHAR);
}

void glWindow::addEmpty(void) {		// adds empty base to the end
	qglColor(CURRENTCL);
	glCallList(EMPTYBASE);
}

void glWindow::paintGL(void) {			// paints the OpenGL scene
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// clears color and depth buffer
	callScene();		// signal for painting the scene (connected in window.cpp)
}

void glWindow::resizeGL( int w, int h ) {		// paint the OpenGL if window size is changed
	glMatrixMode( GL_PROJECTION );		// loads the projection matrix ...
	glLoadIdentity();						// ... and fills with identity matrix
	glViewport(0,0,w,h);					// selects the whole widget as paintable
	glMatrixMode( GL_MODELVIEW );		// loads the model matrix
}

void glWindow::initializeGL(void) {			// inits the OpenGL scene

	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();						// auf GL_PROJECTION
	glOrtho( -1.0, +1.0, -1.0, +1.0, -100.0, +100.0 );		// sets the clipping planes at each six sides
	glViewport(0,0,width(),height());		// selects the whole widget as paintable

	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();					// auf GL_MODELVIEW

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	// clears color and depth buffer
	glClearColor(0.0F,0.0F,0.0F,0.0F);	// fills the window with black color

	glEnable(GL_DEPTH_TEST);		// z-buffer test
	glDepthFunc(GL_LEQUAL);			// paints only objects which are in front or at the same distance like the rest

	glEnable(GL_CULL_FACE);			// provides painting too much
	glFrontFace(GL_CCW);		// sets the rotation mode to counterclockwise
	glCullFace(GL_BACK);		// back side of polygons are not painted

	// glDisable(GL_LIGHTING);			// Lighting is off
	glEnable(GL_COLOR_MATERIAL);		// material color is defined through glColor or qglColor

	genObjects();		// generates all objects
}

void glWindow::genObjects(void) {		// generates all objects

	glNewList(ACHAR, GL_COMPILE);		// generates the character for Adenin
	   qglColor(white);
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.06,0.4,0.0);
		glVertex3d(0.1,0.4,0.0);
		glVertex3d(0.04,0.6,0.0);
		glVertex3d(0.0,0.6,0.0);
		glVertex3d(0.0,0.56,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.0,0.56,0.0);
		glVertex3d(0.0,0.6,0.0);
		glVertex3d(-0.04,0.6,0.0);
		glVertex3d(-0.1,0.4,0.0);
		glVertex3d(-0.06,0.4,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.05,0.46,0.0);
		glVertex3d(0.05,0.5,0.0);
		glVertex3d(-0.05,0.5,0.0);
		glVertex3d(-0.05,0.46,0.0);
	   glEnd();
	glEndList();

	glNewList(CCHAR, GL_COMPILE);		// generates the character for Cytosin
	   qglColor(white);
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.04,0.54,0.0);
		glVertex3d(0.08,0.54,0.0);
		glVertex3d(0.08,0.56,0.0);
		glVertex3d(0.04,0.6,0.0);
		glVertex3d(0.02,0.56,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.02,0.56,0.0);
		glVertex3d(0.04,0.6,0.0);
		glVertex3d(-0.04,0.6,0.0);
		glVertex3d(-0.02,0.56,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.02,0.56,0.0);
		glVertex3d(-0.04,0.6,0.0);
		glVertex3d(-0.1,0.54,0.0);
		glVertex3d(-0.06,0.52,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.06,0.52,0.0);
		glVertex3d(-0.1,0.54,0.0);
		glVertex3d(-0.1,0.46,0.0);
		glVertex3d(-0.06,0.48,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.06,0.48,0.0);
		glVertex3d(-0.1,0.46,0.0);
		glVertex3d(-0.04,0.4,0.0);
		glVertex3d(-0.02,0.44,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.02,0.44,0.0);
		glVertex3d(-0.04,0.4,0.0);
		glVertex3d(0.04,0.4,0.0);
		glVertex3d(0.02,0.44,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.02,0.44,0.0);
		glVertex3d(0.04,0.4,0.0);
		glVertex3d(0.08,0.44,0.0);
		glVertex3d(0.08,0.46,0.0);
		glVertex3d(0.04,0.46,0.0);
	   glEnd();
	glEndList();

	glNewList(GCHAR, GL_COMPILE);		// generates the character for Guanin
	   qglColor(white);
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.04,0.54,0.0);
		glVertex3d(0.08,0.54,0.0);
		glVertex3d(0.08,0.56,0.0);
		glVertex3d(0.04,0.6,0.0);
		glVertex3d(0.02,0.56,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.02,0.56,0.0);
		glVertex3d(0.04,0.6,0.0);
		glVertex3d(-0.04,0.6,0.0);
		glVertex3d(-0.02,0.56,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.02,0.56,0.0);
		glVertex3d(-0.04,0.6,0.0);
		glVertex3d(-0.1,0.54,0.0);
		glVertex3d(-0.06,0.52,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.06,0.52,0.0);
		glVertex3d(-0.1,0.54,0.0);
		glVertex3d(-0.1,0.46,0.0);
		glVertex3d(-0.06,0.48,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.06,0.48,0.0);
		glVertex3d(-0.1,0.46,0.0);
		glVertex3d(-0.04,0.4,0.0);
		glVertex3d(-0.02,0.44,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.02,0.44,0.0);
		glVertex3d(-0.04,0.4,0.0);
		glVertex3d(0.04,0.4,0.0);
		glVertex3d(0.08,0.44,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.08,0.44,0.0);
		glVertex3d(0.08,0.51,0.0);
		glVertex3d(0.04,0.51,0.0);
		glVertex3d(0.04,0.44,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.04,0.47,0.0);
		glVertex3d(0.04,0.51,0.0);
		glVertex3d(-0.02,0.51,0.0);
		glVertex3d(-0.02,0.47,0.0);
	   glEnd();
	glEndList();

	glNewList(TCHAR, GL_COMPILE);		// generates the character for Thymin
	   qglColor(white);
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.02,0.4,0.0);
		glVertex3d(0.02,0.6,0.0);
		glVertex3d(-0.02,0.6,0.0);
		glVertex3d(-0.02,0.4,0.0);
	   glEnd();
	   glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.1,0.56,0.0);
		glVertex3d(0.1,0.6,0.0);
		glVertex3d(-0.1,0.6,0.0);
		glVertex3d(-0.1,0.56,0.0);
	   glEnd();
	glEndList();

	glNewList(AROTCHAR, GL_COMPILE);		// generates the character for Adenin (rotated)
		glTranslate(0.0,1.0);
		glRotateHalf();
		glCallList(ACHAR);
		glRotateHalf();
		glTranslate(0.0,-1.0);
	glEndList();

	glNewList(CROTCHAR, GL_COMPILE);		// generates the character for Cytosin (rotated)
		glTranslate(0.0,1.0);
		glRotateHalf();
		glCallList(CCHAR);
		glRotateHalf();
		glTranslate(0.0,-1.0);
	glEndList();

	glNewList(GROTCHAR, GL_COMPILE);		// generates the character for Guanin (rotated)
		glTranslate(0.0,1.0);
		glRotateHalf();
		glCallList(GCHAR);
		glRotateHalf();
		glTranslate(0.0,-1.0);
	glEndList();

	glNewList(TROTCHAR, GL_COMPILE);		// generates the character for Thymin (rotated)
		glTranslate(0.0,1.0);
		glRotateHalf();
		glCallList(TCHAR);
		glRotateHalf();
		glTranslate(0.0,-1.0);
	glEndList();


	glNewList(REND, GL_COMPILE);		// generates right yellow connection
	    glBegin(GL_POLYGON);
		qglColor(yellow);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,0.6,0.0);
		glVertex3d(0.3,0.6,0.0);
		glVertex3d(0.35,0.65,0.0);
		glVertex3d(0.3,0.7,0.0);
		glVertex3d(0.35,0.75,0.0);
		glVertex3d(0.3,0.8,0.0);
		glVertex3d(0.2,0.8,0.0);
	   glEnd();
	glEndList();

	glNewList(LEND, GL_COMPILE);		// generates left yellow connection
	    glBegin(GL_POLYGON);
		qglColor(yellow);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.2,0.6,0.0);
		glVertex3d(-0.2,0.8,0.0);
		glVertex3d(-0.35,0.8,0.0);
		glVertex3d(-0.3,0.75,0.0);
		glVertex3d(-0.35,0.7,0.0);
		glVertex3d(-0.3,0.65,0.0);
		glVertex3d(-0.35,0.6,0.0);
	   glEnd();
	glEndList();

	glNewList(ADENIN, GL_COMPILE);		// generates Adenin base

	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,-0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(0.0,0.9,0.0);
		glVertex3d(0.0,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glVertex3d(0.0,0.1,0.0);
		glVertex3d(0.0,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,-0.1,0.0);
	    glEnd();
	    glBegin(GL_LINE_LOOP);		// frame around base
		glLineWidth(3.0);
		qglColor(white);
		glVertex3d(0.2,-0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,-0.1,0.0);
		glVertex3d(0.0,0.1,0.0);
	    glEnd();
	    glCallList(REND);
	    glCallList(LEND);
	glEndList();

	glNewList(CYTOSIN, GL_COMPILE);		// generates Cytosin base
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,0.1,0.0);
		glVertex3d(-0.19,0.05,0.0);
		glVertex3d(-0.17,0.0,0.0);
		glVertex3d(-0.14,-0.04,0.0);
		glVertex3d(-0.1,-0.07,0.0);
		glVertex3d(-0.05,-0.09,0.0);
		glVertex3d(0.0,-0.1,0.0);	// zenith
		glVertex3d(0.05,-0.09,0.0);
		glVertex3d(0.1,-0.07,0.0);
		glVertex3d(0.14,-0.04,0.0);
		glVertex3d(0.17,0.0,0.0);
		glVertex3d(0.19,0.05,0.0);
	    glEnd();
	    glBegin(GL_LINE_LOOP);		// frame around base
		glLineWidth(3.0);
		qglColor(white);
		glVertex3d(0.2,0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,0.1,0.0);
		glVertex3d(-0.19,0.05,0.0);
		glVertex3d(-0.17,0.0,0.0);
		glVertex3d(-0.14,-0.04,0.0);
		glVertex3d(-0.1,-0.07,0.0);
		glVertex3d(-0.05,-0.09,0.0);
		glVertex3d(0.0,-0.1,0.0);	// zenith
		glVertex3d(0.05,-0.09,0.0);
		glVertex3d(0.1,-0.07,0.0);
		glVertex3d(0.14,-0.04,0.0);
		glVertex3d(0.17,0.0,0.0);
		glVertex3d(0.19,0.05,0.0);
	    glEnd();
	    glCallList(REND);
	    glCallList(LEND);
	glEndList();

	glNewList(GUANIN, GL_COMPILE);		// generates Guanin base
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.2,-0.1,0.0);
		glVertex3d(-0.19,-0.05,0.0);
		glVertex3d(-0.2,-0.05,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.2,-0.05,0.0);
		glVertex3d(-0.19,-0.05,0.0);
		glVertex3d(-0.17,0.0,0.0);
		glVertex3d(-0.2,0.0,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.2,0.1,0.0);
		glVertex3d(-0.2,0.0,0.0);
		glVertex3d(-0.17,0.0,0.0);
		glVertex3d(-0.14,0.04,0.0);
		glVertex3d(-0.14,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.14,0.1,0.0);
		glVertex3d(-0.14,0.04,0.0);
		glVertex3d(-0.1,0.07,0.0);
		glVertex3d(-0.1,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.1,0.1,0.0);
		glVertex3d(-0.1,0.07,0.0);
		glVertex3d(-0.05,0.09,0.0);
		glVertex3d(-0.05,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(-0.05,0.1,0.0);
		glVertex3d(-0.05,0.09,0.0);
		glVertex3d(0.0,0.1,0.0);	// zenith
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.0,0.1,0.0);	// zenith
		glVertex3d(0.05,0.09,0.0);
		glVertex3d(0.05,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.05,0.1,0.0);
		glVertex3d(0.05,0.09,0.0);
		glVertex3d(0.1,0.07,0.0);
		glVertex3d(0.1,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.1,0.1,0.0);
		glVertex3d(0.1,0.07,0.0);
		glVertex3d(0.14,0.04,0.0);
		glVertex3d(0.14,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.14,0.1,0.0);
		glVertex3d(0.14,0.04,0.0);
		glVertex3d(0.17,0.0,0.0);
		glVertex3d(0.2,0.0,0.0);
		glVertex3d(0.2,0.1,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,0.0,0.0);
		glVertex3d(0.17,0.0,0.0);
		glVertex3d(0.19,-0.05,0.0);
		glVertex3d(0.2,-0.05,0.0);
	    glEnd();
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,-0.05,0.0);
		glVertex3d(0.19,-0.05,0.0);
		glVertex3d(0.2,-0.1,0.0);
	    glEnd();
	    glBegin(GL_LINE_LOOP);		// frame around base
		glLineWidth(3.0);
		qglColor(white);
		glVertex3d(0.2,-0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,-0.1,0.0);
		glVertex3d(-0.19,-0.05,0.0);
		glVertex3d(-0.17,0.0,0.0);
		glVertex3d(-0.14,0.04,0.0);
		glVertex3d(-0.1,0.07,0.0);
		glVertex3d(-0.05,0.09,0.0);
		glVertex3d(0.0,0.1,0.0);	// zenith
		glVertex3d(0.05,0.09,0.0);
		glVertex3d(0.1,0.07,0.0);
		glVertex3d(0.14,0.04,0.0);
		glVertex3d(0.17,0.0,0.0);
		glVertex3d(0.19,-0.05,0.0);
	    glEnd();
	    glCallList(REND);
	    glCallList(LEND);
	glEndList();

	glNewList(THYMIN, GL_COMPILE);		// generates Thymin base
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,0.1,0.0);
		glVertex3d(0.0,-0.1,0.0);
	    glEnd();
	    glBegin(GL_LINE_LOOP);		// frame around base
		glLineWidth(2.0);
		qglColor(white);
		glVertex3d(0.2,0.1,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,0.1,0.0);
		glVertex3d(0.0,-0.1,0.0);
	    glEnd();
	    glCallList(REND);
	    glCallList(LEND);
	glEndList();

	glNewList(EMPTYBASE, GL_COMPILE);		// generates empty base
	    glBegin(GL_POLYGON);
		glNormal3d(0.0,0.0,1.0);
		glVertex3d(0.2,0.0,0.0);
		glVertex3d(0.2,0.9,0.0);
		glVertex3d(-0.2,0.9,0.0);
		glVertex3d(-0.2,0.0,0.0);
	    glEnd();
	glEndList();


}
