/* DNA-View - window.cpp
Here you find all declarations for the GUI */

/**
Copyright (C) 2003, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "window.h"

#define BUTTON_WIDTH 22		// defines the standard buttons size for the base pairs
#define LABEL_HEIGHT 20		// defines the standard height for all labels and sliders

#define ZOOM_IN_ITEM 21		// definitions for the activate / deactivate menu items
#define ZOOM_OUT_ITEM 22
#define COPY_ITEM 31
#define CUT_ITEM 32

#define FROM_NOTHING 0		// definitions for find from...
#define FROM_CLIPBOARD 1
#define FROM_FILE 2
#define FROM_SELECTION 3
#define FROM_CREATE 4

#define FONT "Courier [Adobe]"		// right font for Solaris
// #define FONT "Courier 10 Pitch"		// right font for Linux

dnadstring dnaClipBoard;		// clipboard to copy a DNA string to
dnadstring searchDNA;		// DNA to search for
dnadstring undoDNA;		// last DNA before changes done for undo

const QString version="DNA-View v1.3";		// shows current version in top bar (declared in dna.h)

/***************************************************************************************/

MainMenu::MainMenu(QWidget *parent, const char *name) : QWidget (parent, name) {		// standard constructor
	menuBar=new QMenuBar(this);
	Q_CHECK_PTR(menuBar);

	QPopupMenu *fileMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(fileMenu);

	fileMenu->insertItem ("&New", this, SLOT(newdna()), CTRL+Key_N);
	fileMenu->insertItem ("&Open...", this, SLOT(openin()), CTRL+Key_O);
	fileMenu->insertSeparator();
	fileMenu->insertSeparator();
	fileMenu->insertItem ("&Quit", qApp, SLOT(closeAllWindows()), CTRL+Key_Q);		// Quit only closes all windows

	connect(qApp, SIGNAL(lastWindowClosed()), qApp, SLOT(quit()));		// if all windows are closed then the program quits

	QPopupMenu *helpMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(helpMenu);
	helpMenu->insertItem ("&Help", this, SLOT(help()),Key_F1);
	helpMenu->insertSeparator();
	helpMenu->insertItem ("&About DNA-View", this, SLOT(about()),SHIFT+Key_F1);
	helpMenu->insertItem ("&About Qt", this, SLOT(aboutqt()));

	menuBar->insertItem("&File",fileMenu);
	menuBar->insertItem("&Help",helpMenu);

	QHBoxLayout *mainLayout=new QHBoxLayout(this, 1);
	Q_CHECK_PTR(mainLayout);
	mainLayout->setMenuBar(menuBar);
}

MainMenu::~MainMenu() {		// destructor
	delete menuBar;
}

void MainMenu::newdna(bool open) {		// creates new window, if open is TRUE then it also opens a file
	dnaMenu *m=new dnaMenu();
	Q_CHECK_PTR(m);
	m->setMinimumSize(m->sizeHint());	// sets the minimum size s.t. all objects are visible
	m->resize(600,400);				// resizes the window to 600x400 pixel
	m->setCaption(version + " - " + m->getFileName());		// sets the window title
	m->show();
	if (open) {			// opens a DNA file if open is TRUE
		m->opendna();
		if (m->getFileName()=="Unknown") m->close();	// if opendna() fails the window is closed again
	}
}

void MainMenu::openin(void) {	// opens file with DNA in new window
	newdna(TRUE);
}

void MainMenu::aboutqt() {		// shows an aboutQt message
    QMessageBox::aboutQt(this, "Qt");
}

void MainMenu::about() {		// shows an about message
    QMessageBox::about(this, version, "DNA-View written by\nDominik Wagenfuehr\nUniversity of Giessen, Germany\nin 2003.");
}

void MainMenu::help() {	// opens the help file in mozilla
	QString helpfile("file://"+QDir::currentDirPath()+"/help/index.htm");

	if (system("mozilla -remote 'openFile("+helpfile+",new-window)' > /dev/null 2>&1"))
	system("mozilla '"+helpfile+"' &");				// opens a new instant of mozilla, I think
}

/******************************************************************************************************************************/

MainWindow::MainWindow (QWidget *parent, const char *name, bool modal) : QDialog (parent,name,modal) {	// standard constructor
	dnaBox=new QTextEdit(this);		// box with the DNA in text mode
	Q_CHECK_PTR(dnaBox);
	dnaBox->setReadOnly(TRUE);
	dnaBox->setWrapPolicy(QTextEdit::Anywhere);
	dnaBox->setFrameStyle(QFrame::Panel | QFrame::Plain);
	dnaBox->setLineWidth(1);
	dnaBox->setPaletteBackgroundColor("black");
	dnaBox->setPaletteForegroundColor("white");
	dnaBox->setAlignment(Qt::AlignLeft);
	dnaBox->setMinimumHeight(80);

	QFont font;
	font.setFamily(FONT);
	font.setFixedPitch(TRUE);
	dnaBox->setFont(font);

	glDnaBox=new glWindow(this);		// box with the DNA in graphic mode
	Q_CHECK_PTR(glDnaBox);
	glDnaBox->setMinimumHeight(80);

	/* the following lines declares all buttons s.t. the user can edit the DNA
		and connects them with the right slots */

	fGroup=new QButtonGroup(4, Qt::Horizontal, "5'-3' Bases",this);
	Q_CHECK_PTR(fGroup);
	fGroup->setExclusive(TRUE);

	tGroup=new QButtonGroup(4, Qt::Horizontal, "3'-5' Bases",this);
	Q_CHECK_PTR(tGroup);
	tGroup->setExclusive(TRUE);

	aGroup=new QButtonGroup(4, Qt::Horizontal, "all Bases",this);
	Q_CHECK_PTR(aGroup);
	aGroup->setExclusive(TRUE);

	butA0=new QPushButton("A\n ",fGroup);
	Q_CHECK_PTR(butA0);
	butA0->setFixedWidth(BUTTON_WIDTH);
	butA0->setAutoRepeat(TRUE);

	butC0=new QPushButton("C\n ",fGroup);
	Q_CHECK_PTR(butC0);
	butC0->setFixedWidth(BUTTON_WIDTH);
	butC0->setAutoRepeat(TRUE);

	butG0=new QPushButton("G\n ",fGroup);
	Q_CHECK_PTR(butG0);
	butG0->setFixedWidth(BUTTON_WIDTH);
	butG0->setAutoRepeat(TRUE);

	butT0=new QPushButton("T\n ",fGroup);
	Q_CHECK_PTR(butT0);
	butT0->setFixedWidth(BUTTON_WIDTH);
	butT0->setAutoRepeat(TRUE);

	but0A=new QPushButton(" \nA",tGroup);
	Q_CHECK_PTR(but0A);
	but0A->setFixedWidth(BUTTON_WIDTH);
	but0A->setAutoRepeat(TRUE);

	but0C=new QPushButton(" \nC",tGroup);
	Q_CHECK_PTR(but0C);
	but0C->setFixedWidth(BUTTON_WIDTH);
	but0C->setAutoRepeat(TRUE);

	but0G=new QPushButton(" \nG",tGroup);
	Q_CHECK_PTR(but0G);
	but0G->setFixedWidth(BUTTON_WIDTH);
	but0G->setAutoRepeat(TRUE);

	but0T=new QPushButton(" \nT",tGroup);
	Q_CHECK_PTR(but0T);
	but0T->setFixedWidth(BUTTON_WIDTH);
	but0T->setAutoRepeat(TRUE);

	butAT=new QPushButton("A\nT",aGroup);
	Q_CHECK_PTR(butAT);
	butAT->setFixedWidth(BUTTON_WIDTH);
	butAT->setAutoRepeat(TRUE);

	butCG=new QPushButton("C\nG",aGroup);
	Q_CHECK_PTR(butCG);
	butCG->setFixedWidth(BUTTON_WIDTH);
	butCG->setAutoRepeat(TRUE);

	butGC=new QPushButton("G\nC",aGroup);
	Q_CHECK_PTR(butGC);
	butGC->setFixedWidth(BUTTON_WIDTH);
	butGC->setAutoRepeat(TRUE);

	butTA=new QPushButton("T\nA",aGroup);
	Q_CHECK_PTR(butTA);
	butTA->setFixedWidth(BUTTON_WIDTH);
	butTA->setAutoRepeat(TRUE);

	connect(butA0, SIGNAL(clicked()), this, SLOT(insertA0()));
	connect(butC0, SIGNAL(clicked()), this, SLOT(insertC0()));
	connect(butG0, SIGNAL(clicked()), this, SLOT(insertG0()));
	connect(butT0, SIGNAL(clicked()), this, SLOT(insertT0()));
	connect(but0A, SIGNAL(clicked()), this, SLOT(insert0A()));
	connect(but0C, SIGNAL(clicked()), this, SLOT(insert0C()));
	connect(but0G, SIGNAL(clicked()), this, SLOT(insert0G()));
	connect(but0T, SIGNAL(clicked()), this, SLOT(insert0T()));
	connect(butAT, SIGNAL(clicked()), this, SLOT(insertAT()));
	connect(butCG, SIGNAL(clicked()), this, SLOT(insertCG()));
	connect(butGC, SIGNAL(clicked()), this, SLOT(insertGC()));
	connect(butTA, SIGNAL(clicked()), this, SLOT(insertTA()));

	fGroup->setFixedSize(fGroup->sizeHint());
	tGroup->setFixedSize(tGroup->sizeHint());
	aGroup->setFixedSize(aGroup->sizeHint());

	slider=new QSlider(Qt::Horizontal, this);		// slider to scroll left and right through DNA
	Q_CHECK_PTR(slider);
	slider->setLineStep(1);
	slider->setPageStep(10);
	slider->setTracking(TRUE);
	slider->setFixedHeight(LABEL_HEIGHT);

	/* connects the slider with the current position */
	connect(slider,SIGNAL(sliderMoved(int)), this, SLOT(changePos(int)));
	connect(slider,SIGNAL(valueChanged(int)), this, SLOT(changePos(int)));

	sliderCurrent=new QLabel(this);		// label with current position
	Q_CHECK_PTR(sliderCurrent);
	sliderCurrent->setFrameStyle(QFrame::Panel | QFrame::Plain);
	sliderCurrent->setLineWidth(1);
	sliderCurrent->setFixedHeight(LABEL_HEIGHT);
	sliderCurrent->setAlignment(Qt::AlignLeft);

	connect(this, SIGNAL(posChanged()), this, SLOT(updateSliderCurrent()));

	sliderEnd=new QLabel(this);		// label with last position
	Q_CHECK_PTR(sliderEnd);
	sliderEnd->setFrameStyle(QFrame::Panel | QFrame::Plain);
	sliderEnd->setLineWidth(1);
	sliderEnd->setFixedHeight(LABEL_HEIGHT);
	sliderEnd->setAlignment(Qt::AlignRight);

	connect(this, SIGNAL(endChanged(int)), this, SLOT(updateSliderEnd(int)));
	connect(this, SIGNAL(sliderFocus()), this, SLOT(setSliderFocus()));

	insLabel=new QLabel(this);			// label to show if insert or overwrite mode is set
	Q_CHECK_PTR(sliderEnd);
	insLabel->setFrameStyle(QFrame::Panel | QFrame::Plain);
	insLabel->setLineWidth(1);
	insLabel->setFixedSize(75,LABEL_HEIGHT);
	insLabel->setAlignment(Qt::AlignHCenter);
	ins=FALSE;
	updateInsert(); 			// changes from insert to overwrite mode or vice versa

	changePos(1);
	endChanged(pos);
}

MainWindow::~MainWindow() {		// destructor
	delete dnaBox;
	delete slider;
	delete sliderCurrent;
	delete sliderEnd;
}

void MainWindow::resizeEvent (QResizeEvent *) {		// event if user resizes the window
	viewChanged();
}

void MainWindow::keyPressEvent (QKeyEvent *event) {		// event if user press a key (needed for backspace, delete and insert)
	switch(event->key()) {
	    case Key_Backspace: if (pos>1) {
							changePos(pos-1);	// deletes base pair at position-1 and sets pos--
	    						del(pos);
						}
						break;
	    case Key_Delete:	del(pos);			// deletes base pair at position
	    					break;
	    case Key_Insert: 		updateInsert();			// changes from insert to overwrite mode or vice versa
	    					break;
	    default: 			break;
	}
}

void MainWindow::setSliderFocus(void) {		// sets the focus to the slider, s.t. you can move it with the keyboard
	slider->setFocus();
}

void MainWindow::changePos(int n) {			// sets position new if slider has moved
	pos=n;
	posChanged();			// signal that position has changed
	slider->setValue(pos);
}

void MainWindow::updateSliderCurrent() {	// repaints the current position label
	QString temp;
	temp=temp.setNum(pos);
	sliderCurrent->setText("Current Position: "+temp);
}

void MainWindow::updateSliderEnd(int n) {	// repaints the end position label
	end=dna.length()+1;
	QString temp;
	temp=temp.setNum(end);
	sliderEnd->setText("Last Position: "+temp);
	slider->setRange(n,end);	// defines new slider start and end
}

void MainWindow::updateInsert(void) {		// repaints the insert label and change from insert to overwrite mode or vice versa
	if (ins) {
		ins=FALSE;
		insLabel->setText("Overwrite");
	}
	else {
		ins=TRUE;
		insLabel->setText("Insert");
	}
}

void MainWindow::del(uint position) {				// deletes base pair at position
	QApplication::setOverrideCursor(waitCursor);
	if (!dna.del(position)) {
		updateSliderEnd(1);
		dnaChanged(TRUE, TRUE);				// signal that DNA has changed and the marks need to be reset
	}
	QApplication::restoreOverrideCursor();
}

void MainWindow::insert(uint position, char fc, char tc) {		// inserts at position the char fc in fstring and tc in tstring
	QApplication::setOverrideCursor(waitCursor);
	int res=dna.insert(position,fc,tc,ins);
	if (res) {
		QApplication::restoreOverrideCursor();
		if (res==1) {
			QMessageBox messageBox;
			messageBox.information(0,
			version + " - Wrong Insert", "Insertion would lead to an incorrect DNA string.", "&Ok", QString::null, QString::null, 0, 0);
		}
	}
	else {
		updateSliderEnd(1);
		changePos(pos+1);
		dnaChanged(TRUE, TRUE);			// signal that DNA has changed and the marks need to be reset
		QApplication::restoreOverrideCursor();
	}
	viewChanged();
}

/* the following lines insert different base pairs in the dna string at position pos */

void MainWindow::insertA0(void) {
	insert(pos,'A',' ');
}

void MainWindow::insertC0(void) {
	insert(pos,'C',' ');
}

void MainWindow::insertG0(void) {
	insert(pos,'G',' ');
}

void MainWindow::insertT0(void) {
	insert(pos,'T',' ');
}

void MainWindow::insert0A(void) {
	insert(pos,' ','A');
}

void MainWindow::insert0C(void) {
	insert(pos,' ','C');
}

void MainWindow::insert0G(void) {
	insert(pos,' ','G');
}

void MainWindow::insert0T(void) {
	insert(pos,' ','T');
}

void MainWindow::insertAT(void) {
	insert(pos,'A','T');
}

void MainWindow::insertCG(void) {
	insert(pos,'C','G');
}

void MainWindow::insertGC(void) {
	insert(pos,'G','C');
}

void MainWindow::insertTA(void) {
	insert(pos,'T','A');
}

/******************************************************************************************************************************/

dnaMenu::dnaMenu(QWidget *parent, const char *name, bool modal) : MainWindow (parent,name,modal) {	// standard constructor
	menuBar=new QMenuBar(this);
	Q_CHECK_PTR(menuBar);

	QPopupMenu *fileMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(fileMenu);
	fileMenu->insertItem ("&Save", this, SLOT(savedna()), CTRL+Key_S);
	fileMenu->insertItem ("Save &as...", this, SLOT(saveas()), CTRL+Key_U);
	fileMenu->insertSeparator();
	fileMenu->insertItem ("&Close", this, SLOT(closedna()), CTRL+Key_F4);

	QPopupMenu *viewMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(viewMenu);
	viewMenu->insertItem("Toggle Text/&Graphic Mode",this, SLOT(toggleText()),CTRL+Key_G);
	viewMenu->insertSeparator();
	viewMenu->insertItem("Zoom &in",this, SLOT(zoomIn()),Key_Plus, ZOOM_IN_ITEM);
	viewMenu->insertItem("Zoom &out",this, SLOT(zoomOut()),Key_Minus, ZOOM_OUT_ITEM);

	QPopupMenu *editMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(editMenu);
	editMenu->insertItem ("Sets Un&do Point", this, SLOT(setUndo()),CTRL+Key_D);
	editMenu->insertItem ("&Undo last changes", this, SLOT(undo()),CTRL+Key_Z);
	editMenu->insertSeparator();
	editMenu->insertItem ("&Reset all marks", this, SLOT(resetMarks()),CTRL+Key_R);
	editMenu->insertItem ("Reset 5'-3' marks", this, SLOT(resetFMarks()));
	editMenu->insertItem ("Reset 3'-5' marks", this, SLOT(resetTMarks()));
	editMenu->insertSeparator();
	editMenu->insertItem ("Set 5'-3' Start", this, SLOT(setfStart()),CTRL+Key_1);
	editMenu->insertItem ("Set 5'-3' End", this, SLOT(setfEnd()),CTRL+Key_2);
	editMenu->insertItem ("Set 3'-5' Start", this, SLOT(settStart()),CTRL+Key_3);
	editMenu->insertItem ("Set 3'-5' End", this, SLOT(settEnd()),CTRL+Key_4);
	editMenu->insertSeparator();
	editMenu->insertItem ("Mark &all", this, SLOT(markAll()),CTRL+Key_A);
	editMenu->insertSeparator();
	editMenu->insertItem ("&Copy", this, SLOT(copy()),CTRL+Key_C, COPY_ITEM);
	editMenu->insertItem ("Cu&t", this, SLOT(cut()),CTRL+Key_X, CUT_ITEM);
	editMenu->insertItem ("&Paste...", this, SLOT(paste()),CTRL+Key_P);
	editMenu->insertSeparator();
	editMenu->insertItem ("Paste from clipboard", this, SLOT(pasteClip()),CTRL+Key_V);
	editMenu->insertItem ("Paste from file", this, SLOT(pasteFile()));
	editMenu->insertItem ("Paste selection", this, SLOT(pasteSel()));
	editMenu->insertSeparator();
	editMenu->insertSeparator();
	editMenu->insertItem ("&Blunt", this, SLOT(blunt()),CTRL+Key_B);
	editMenu->insertSeparator();
	editMenu->insertSeparator();
	editMenu->insertItem ("&Split", this, SLOT(split()),CTRL+Key_T);

	QPopupMenu *searchMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(searchMenu);
	searchMenu->insertItem ("&Find...", this, SLOT(find()),CTRL+Key_F);
	searchMenu->insertItem ("Find &again", this, SLOT(findAgain()),Key_F3);
	searchMenu->insertSeparator();
	searchMenu->insertItem ("Find from &clipboard", this, SLOT(findClip()));
	searchMenu->insertItem ("Find from f&ile", this, SLOT(findFile()));
	searchMenu->insertItem ("Find from &new DNA", this, SLOT(findNew()));
	searchMenu->insertItem ("Find &selection", this, SLOT(findSel()));

	QPopupMenu *extraMenu=new QPopupMenu(menuBar);
	Q_CHECK_PTR(extraMenu);
	extraMenu->insertItem ("&Statistics", this, SLOT(stats()),Key_F12);

	menuBar->insertItem("&File",fileMenu);
	menuBar->insertItem("&Edit",editMenu);
	menuBar->insertItem("&Search",searchMenu);
	menuBar->insertItem("&View",viewMenu);
	menuBar->insertItem("E&xtras",extraMenu);

	zoomLabel=new QLabel(this);			// label for zoom
	Q_CHECK_PTR(zoomLabel);
	zoomLabel->setFrameStyle(QFrame::Panel | QFrame::Plain);
	zoomLabel->setLineWidth(1);
	zoomLabel->setAlignment(Qt::AlignHCenter);
	zoomLabel->setFixedSize(50,LABEL_HEIGHT);

	connect(glDnaBox, SIGNAL(callScene()), this, SLOT(updateViewGL()));		// update the OpenGL scene if the signal is given

	connect(this, SIGNAL(zoomChanged()), this, SLOT(updateView()));	// every change needs to update the text window
	connect(this, SIGNAL(posChanged()), this, SLOT(updateView()));
	connect(this, SIGNAL(viewChanged()), this, SLOT(updateView()));

	QVBoxLayout *mainLayout=new QVBoxLayout(this, 1);
	Q_CHECK_PTR(mainLayout);

	mainLayout->setMenuBar(menuBar);
	mainLayout->addWidget(dnaBox);		// adds the text DNA box
	glDnaBox->hide();				// hides the graphical DNA box, s.t. only one box at a time is shown
	mainLayout->addWidget(glDnaBox);	// adds the graphical DNA box

	QHBoxLayout *createLayout=new QHBoxLayout(mainLayout);
	Q_CHECK_PTR(createLayout);
	createLayout->addWidget(fGroup);
	createLayout->addWidget(tGroup);
	createLayout->addWidget(aGroup);

	QGridLayout *sliderLayout=new QGridLayout(mainLayout, 2, 4);
	Q_CHECK_PTR(sliderLayout);
	sliderLayout->addMultiCellWidget(slider,0,0,0,3);
	sliderLayout->addWidget(sliderCurrent,1,0);
	sliderLayout->addWidget(zoomLabel,1,1);
	sliderLayout->addWidget(insLabel,1,2);
	sliderLayout->addWidget(sliderEnd,1,3);

	statusBar=new QStatusBar(this);
	Q_CHECK_PTR(statusBar);
	statusBar->setFixedHeight(LABEL_HEIGHT);

	connect(this, SIGNAL(dnaChanged(bool,bool)), this, SLOT (updateChange(bool,bool)));

	mainLayout->addWidget(statusBar);

	zoomStd=(dnaBox->fontMetrics()).width(' ')*1.0;
	updateZoom(1.0);				// sets the zoom factor to 100%
	zoomIn(); zoomIn(); zoomIn();
	textMode=TRUE;				// set text mode on
	toggleText();					// switches to graphic mode
	dnaChanged(FALSE, TRUE);		// DNA has not changed, but the marks need to be reset
	filename="Unknown";			// filename is set to Unknown

}

dnaMenu::~dnaMenu() {		// destructor
	delete menuBar;
	delete zoomLabel;
	delete statusBar;
}

void dnaMenu::setUndo(void) {		// sets Undo Point
	QApplication::setOverrideCursor(waitCursor);
	undoDNA=dna;
	QApplication::restoreOverrideCursor();
	statusBar->message("Undo point set.",DISP_TIME);
}

void dnaMenu::undo(void) {			// restores last changes
	QApplication::setOverrideCursor(waitCursor);
	if (undoDNA.isEmpty()!=BOTH_EMPTY) {
		dna=undoDNA;
		if (pos>dna.length()+1) changePos(dna.length()+1);		// pos is too far away, set pos to end
		endChanged(1);
		dnaChanged(TRUE,TRUE);				// DNA has changed and the marks need to be reset
		statusBar->message("DNA restored.",DISP_TIME);
	}
	QApplication::restoreOverrideCursor();
}

QString dnaMenu::getFileName(void) const {		// returns the current filename (incl. path) the DNA is saved under
	return(filename);
}

void dnaMenu::opendna(void) {		// opens a DNA file and reads the data

	QString fn = QFileDialog::getOpenFileName(QString::null, QString::null, this);
	if (!fn.isEmpty()) {
		QApplication::setOverrideCursor(waitCursor);
		if (!dna.read(fn)) {						// DNA data from file is read and saved
			filename=fn;						// filename is set (incl. path)
			uint i=filename.findRev(QRegExp("[\\/]"));
			setCaption(version + " - " + filename.right(filename.length()-i-1));	// caption is set with filename (excl. path)
			endChanged(1);
			changePos(1);
			dnaChanged(FALSE, TRUE);		// DNA has not changed, but the marks need to be reset
			statusBar->message("File "+filename+" opened.",DISP_TIME);
		}
		QApplication::restoreOverrideCursor();
	}
}

void dnaMenu::savedna(void) {		// saves DNA with current filename

	if (filename!="Unknown") {		// DNA was saved before
		QApplication::setOverrideCursor(waitCursor);
		dna.write(filename);		// writes DNA to file
		QApplication::restoreOverrideCursor();
		statusBar->message("File "+filename+" saved.",DISP_TIME);
		dnaChanged(FALSE,FALSE);	// DNA has not changed and marks need not to be reset
	}
	else saveas();		// DNA wasn't saved before
}

void dnaMenu::saveas(void) {		// saves DNA with new filename
	QString fn;
	QMessageBox messageBox;
	int res=0;

	if (filename=="Unknown") {		// DNA wasn't saved before
		do {
			fn=QFileDialog::getSaveFileName(QString::null, QString::null, this);		// gets the filename user wants to save DNA data in
			if (QFile::exists(fn)) {				// file exists already
				res=messageBox.warning(0,
					version + " - File exists already",
					"The file "+fn+" exists already.\nDo you want to overwrite it?",
        				"&Overwrite", "&Discard", "&Cancel", 1, 2);
			}
		} while (res==1);
		if (res==2) fn="";		// cancel was choosen in "overwrite"-dialog
	}
	else {			// DND was saved before
		do {
			fn=QFileDialog::getSaveFileName(filename, QString::null, this);	// gets the filename user wants to save DNA data in
			if (QFile::exists(fn)) {				// file exists already
				res=messageBox.warning(0,
					version + " - File exists already",
					"The file "+fn+" exists already.\nDo you want to overwrite it?",
        				"&Overwrite", "&Discard", "&Cancel", 1, 2);
			}
		} while (res==1);
		if (res==2) fn="";		// cancel was choosen in "overwrite"-dialog
	}

	if (!fn.isEmpty()) {		// a correct filename had be choosen to save
		QApplication::setOverrideCursor(waitCursor);
		dna.write(fn);		// writes the DNA to file
		filename=fn;		// sets the new filename (incl. path)
		uint i=filename.findRev(QRegExp("[\\/]"));
		setCaption(version + " - "+ filename.right(filename.length()-i-1));	// caption is set with filename (excl. path)
		dnaChanged(FALSE,FALSE);			// DNA has not changed and marks need not to be reset
		QApplication::restoreOverrideCursor();
		statusBar->message("File "+filename+" saved.",DISP_TIME);
	}
}

void dnaMenu::closedna(void) {		// closes the window
	int res=1;
	if (changed) {			// DNA has changed but not saved
		QMessageBox messageBox;
		res=messageBox.warning(0,
			version + " - File not saved",
			"The file has changed and is not saved yet.\nDo you want to save it now?",
        		"&Save", "&Discard", "&Cancel", 0, 2);
	}
	if (res==2) return;		// user does not want to close the window anymore
	if (res==0) savedna();		// user wants to save the DNA before leeaving
	close();				// closes the window
}

void dnaMenu::cut(void) {				// cuts a dna string at marks out

	if (dna.isEmpty()!=BOTH_EMPTY) {		// DNA string is not empty at all (there is something to cut)
		QApplication::setOverrideCursor(waitCursor);
		if (!dna.cut(fstart,fend,tstart,tend)) {		// tries to cut the DNA

			statusBar->message("DNA string cut.",DISP_TIME);
			if (pos<MIN(fstart,tstart)) changePos(1);		// cut was too far right
			else changePos(pos-MIN(fstart,tstart)+1);	// new position after cut "is" old position before cut
			if (pos>dna.length()+1) changePos(dna.length()+1);		// pos is too far away, set pos to end
			endChanged(1);
			dnaChanged(TRUE,TRUE);		// DNA has changed and the marks need to be reset
		}
		else statusBar->message("DNA cut is not correct.",DISP_TIME);
		QApplication::restoreOverrideCursor();
	}
}

void dnaMenu::copy(void) {					// copy a dna string at marks to clipboard

	if (dna.isEmpty()!=BOTH_EMPTY) {		// DNA string is not empty at all (there is something to copy)
		QApplication::setOverrideCursor(waitCursor);
		dnadstring ds;
		ds=dna;
		if (!ds.cut(fstart,fend,tstart,tend)) {		// checks, if  "copy" is legal
			statusBar->message("DNA string copied.",DISP_TIME);
			dnaClipBoard=ds;
		}
		else statusBar->message("DNA copy is not correct.",DISP_TIME);
		QApplication::restoreOverrideCursor();
	}
}

void dnaMenu::blunt(void) {		// blunts a DNA string
	if (dna.isEmpty()!=3) {		// only blunts if one of two single strings is not empty
		QApplication::setOverrideCursor(waitCursor);
		dna.blunt();
		if (pos>dna.length()+1) changePos(dna.length()+1);		// pos is too far away, set pos to end
		endChanged(1);
		dnaChanged(TRUE,TRUE);				// DNA has changed and the marks need to be reset
		QApplication::restoreOverrideCursor();
		statusBar->message("DNA string is now blunt.",DISP_TIME);
	}
}

void dnaMenu::split(void) {		// splits DNA double string in two single strings
	if (!dna.isEmpty()) {		// only split if one of two single strings is not empty
		QApplication::setOverrideCursor(waitCursor);

		dnadstring ds1;
		ds1=dna;
		dna.cut(1,dna.length(),0,0);
		dna.fit();
		ds1.cut(0,0,1,ds1.length());
		ds1.fit();
		dnaChanged(TRUE,TRUE);				// DNA has changed and the marks need to be reset

		dnaMenu *m=new dnaMenu();
		Q_CHECK_PTR(m);
		m->setMinimumSize(m->sizeHint());	// sets the minimum size s.t. all objects are visible
		m->resize(600,400);				// resizes the window to 600x400 pixel
		m->setCaption(version + " - " + m->getFileName());		// sets the window title
		m->dna=ds1;
		m->endChanged(1);
		m->dnaChanged(TRUE,TRUE);				// DNA has changed and the marks need to be reset
		m->show();

		QApplication::restoreOverrideCursor();
	}
}

void dnaMenu::paste(int from) {
	int res;
	dnadstring pasteDNA;

	if (from==FROM_NOTHING) {		// really new paste
		findMenu *m=new findMenu(this, QString::null, TRUE, TRUE);
		Q_CHECK_PTR(m);
		m->setFixedSize(m->sizeHint());
		m->setCaption(version + " - Paste DNA");
		res=m->exec();
	}
	else res=from;
	switch (res) {
	   case FROM_CLIPBOARD: 	pasteDNA=dnaClipBoard; break;		// pastes the DNA in the clipboard
	   case FROM_FILE: {
	   		QString fn = QFileDialog::getOpenFileName(QString::null, QString::null, this);		// opens a DNA file
			if (!fn.isEmpty()) {
				QApplication::setOverrideCursor(waitCursor);
				pasteDNA.read(fn);		// save the file data in DNA to paste
				QApplication::restoreOverrideCursor();
			}
			else return;
			break;
			}
	   case FROM_SELECTION: 				// pastes the current selection
	   			QApplication::setOverrideCursor(waitCursor);
				pasteDNA=dna;
				if (pasteDNA.cut(fstart,fend,tstart,tend)) pasteDNA.set("","");
				QApplication::restoreOverrideCursor();
				break;
	   default: 	return; break;
	}

	QApplication::setOverrideCursor(waitCursor);
	res=(dna+=pasteDNA);
	QApplication::restoreOverrideCursor();
	QMessageBox messageBox;

	switch (res) {
		case 0:	endChanged(1);
				dnaChanged(TRUE,FALSE);				// DNA has changed and the marks need not to be reset
				viewChanged();
				statusBar->message("DNA string paste.",DISP_TIME);
				break;
		case 1:	messageBox.warning(0,
					version + " - Concatenate DNA strings",
					"Concatenation would lead to an incorrect DNA string.",
					"&Ok", QString::null, QString::null, 0, 0);
				break;
		default:	break;
	}
}

void dnaMenu::pasteClip(void) {		// pastes the DNA in the clipboard
	paste(FROM_CLIPBOARD);
}

void dnaMenu::pasteFile(void) {		// opens a file and pastes this to the DNA
	paste(FROM_FILE);
}

void dnaMenu::pasteSel(void) {		// pastes the current selection
	paste(FROM_SELECTION);
}

void dnaMenu::stats(void) {			// shows a statistic dialog

	if (dna.isEmpty()==BOTH_EMPTY) {
		QMessageBox messageBox;
		messageBox.information(0,
			version + " - Statistic",
			"There is no DNA string to show some statistics.",
			"&Ok", QString::null, QString::null, 0, 0);
		return;
	}
	QDialog *statWindow=new QDialog(this,QString::null,TRUE);
	Q_CHECK_PTR(statWindow);

	QString temp;
	QApplication::setOverrideCursor(waitCursor);

	uint rlen=dna.length(), len=dna.rlength(), flen=dna.rlengthf(), tlen=dna.rlengtht(),
	     nA=dna.count('A'), nC=dna.count('C'), nG=dna.count('G'), nT=dna.count('T'),
	     nAT=dna.count('A','T'), nCG=dna.count('C','G'), nGC=dna.count('G','C'),
	     nTA=dna.count('T','A');
	uint nAB=nA+nC+nG+nT, nAP=nAT+nCG+nGC+nTA;

	QLabel *t1=new QLabel("Length of the DNA string\n(number of base pairs):",statWindow);
	QLabel *t2=new QLabel("Real length of the DNA string:",statWindow);
	QLabel *t3=new QLabel("Length of the 5'-3' string (number of bases):   ",statWindow);
	QLabel *t4=new QLabel("Length of the 3'-5' string (number of bases):   ",statWindow);
	QLabel *t6=new QLabel("all bases:",statWindow);
	QLabel *t7=new QLabel("Adenin (A) bases:",statWindow);
	QLabel *t8=new QLabel("Cytosin (C) bases :",statWindow);
	QLabel *t9=new QLabel("Guanin (G) bases:",statWindow);
	QLabel *t10=new QLabel("Thymin (T) bases:",statWindow);
	QLabel *t11=new QLabel("\nall base pairs:",statWindow);
	QLabel *t12=new QLabel("A-T base pairs:",statWindow);
	QLabel *t13=new QLabel("C-G base pairs:",statWindow);
	QLabel *t14=new QLabel("G-C base pairs:",statWindow);
	QLabel *t15=new QLabel("T-A base pairs:\n",statWindow);
	QLabel *t5a=new QLabel("\nNumber of...:",statWindow);
	QLabel *t5b=new QLabel("\nabsolut",statWindow);
	QLabel *t5c=new QLabel("\nrelativ",statWindow);

	QLabel *z1=new QLabel(temp.setNum(len),statWindow);
	QLabel *z2=new QLabel(temp.setNum(rlen),statWindow);
	QLabel *z3=new QLabel(temp.setNum(flen),statWindow);
	QLabel *z4=new QLabel(temp.setNum(tlen),statWindow);
	QLabel *z6a=new QLabel(temp.setNum(nAB),statWindow);
	QLabel *z6b=new QLabel(temp.setNum(100.0*nAB/nAB)+"%   ",statWindow);
	QLabel *z7a=new QLabel(temp.setNum(nA),statWindow);
	QLabel *z7b=new QLabel(temp.setNum(100.0*nA/nAB)+" %    ",statWindow);
	QLabel *z8a=new QLabel(temp.setNum(nC),statWindow);
	QLabel *z8b=new QLabel(temp.setNum(100.0*nC/nAB)+" %    ",statWindow);
	QLabel *z9a=new QLabel(temp.setNum(nG),statWindow);
	QLabel *z9b=new QLabel(temp.setNum(100.0*nG/nAB)+" %    ",statWindow);
	QLabel *z10a=new QLabel(temp.setNum(nT),statWindow);
	QLabel *z10b=new QLabel(temp.setNum(100.0*nT/nAB)+" %    ",statWindow);
	QLabel *z11a=new QLabel("\n"+temp.setNum(nAP),statWindow);
	QLabel *z11b;
	if (nAP) z11b=new QLabel("\n"+temp.setNum(100.0*nAP/nAP)+"%    ",statWindow);
	else z11b=new QLabel("\n0%    ",statWindow);
	QLabel *z12a=new QLabel(temp.setNum(nAT),statWindow);
	QLabel *z12b;
	if (nAP) z12b=new QLabel(temp.setNum(100.0*nAT/nAP)+" %    ",statWindow);
	 else z12b=new QLabel("0%    ",statWindow);
	QLabel *z13a=new QLabel(temp.setNum(nCG),statWindow);
	QLabel *z13b;
	if (nAP) z13b=new QLabel(temp.setNum(100.0*nCG/nAP)+" %    ",statWindow);
	else  z13b=new QLabel("0%    ",statWindow);
	QLabel *z14a=new QLabel(temp.setNum(nGC),statWindow);
	QLabel *z14b;
	if (nAP) z14b=new QLabel(temp.setNum(100.0*nGC/nAP)+" %    ",statWindow);
	else z14b=new QLabel("0%    ",statWindow);
	QLabel *z15a=new QLabel(temp.setNum(nTA)+"\n",statWindow);
	QLabel *z15b;
	if (nAP) z15b=new QLabel(temp.setNum(100.0*nTA/nAP)+" %    \n",statWindow);
	else  z15b=new QLabel("0%    \n",statWindow);

	QPushButton *but=new QPushButton("&Ok",statWindow);
	Q_CHECK_PTR(but);
	but->setFixedSize(80,25);
	but->setDefault(TRUE);

	QGridLayout *statLayout=new QGridLayout(statWindow, 16, 3, 3);
	Q_CHECK_PTR(statLayout);
	statLayout->addWidget(t1,0,0);
	statLayout->addWidget(t2,1,0);
	statLayout->addWidget(t3,2,0);
	statLayout->addWidget(t4,3,0);
	statLayout->addWidget(t6,5,0);
	statLayout->addWidget(t7,6,0);
	statLayout->addWidget(t8,7,0);
	statLayout->addWidget(t9,8,0);
	statLayout->addWidget(t10,9,0);
	statLayout->addWidget(t11,10,0);
	statLayout->addWidget(t12,11,0);
	statLayout->addWidget(t13,12,0);
	statLayout->addWidget(t14,13,0);
	statLayout->addWidget(t15,14,0);
	statLayout->addWidget(t5a,4,0);
	statLayout->addWidget(t5b,4,1);
	statLayout->addWidget(t5c,4,2);

	statLayout->addMultiCellWidget(z1,0,0,1,2);
	statLayout->addMultiCellWidget(z2,1,1,1,2);
	statLayout->addMultiCellWidget(z3,2,2,1,2);
	statLayout->addMultiCellWidget(z4,3,3,1,2);
	statLayout->addWidget(z6a,5,1);
	statLayout->addWidget(z6b,5,2);
	statLayout->addWidget(z7a,6,1);
	statLayout->addWidget(z7b,6,2);
	statLayout->addWidget(z8a,7,1);
	statLayout->addWidget(z8b,7,2);
	statLayout->addWidget(z9a,8,1);
	statLayout->addWidget(z9b,8,2);
	statLayout->addWidget(z10a,9,1);
	statLayout->addWidget(z10b,9,2);
	statLayout->addWidget(z11a,10,1);
	statLayout->addWidget(z11b,10,2);
	statLayout->addWidget(z12a,11,1);
	statLayout->addWidget(z12b,11,2);
	statLayout->addWidget(z13a,12,1);
	statLayout->addWidget(z13b,12,2);
	statLayout->addWidget(z14a,13,1);
	statLayout->addWidget(z14b,13,2);
	statLayout->addWidget(z15a,14,1);
	statLayout->addWidget(z15b,14,2);


	statLayout->addWidget(but,16,1);

	connect(but, SIGNAL(clicked()), statWindow, SLOT(accept()));

	uint i=filename.findRev(QRegExp("[\\/]"));
	statWindow->setCaption(version + " - Statistic for "+ filename.right(filename.length()-i-1));
	statWindow->setFixedSize(statWindow->sizeHint());

	QApplication::restoreOverrideCursor();

	statWindow->exec();		// shows the statistic window
}

void dnaMenu::resetMarks(void) {		// resets all marks
	fstart=0; fend=0;
	tstart=0; tend=0;
	menuBar->setItemEnabled(COPY_ITEM, FALSE);		// disables "copy"
	menuBar->setItemEnabled(CUT_ITEM, FALSE);		// disables "cut"
	viewChanged();
}

void dnaMenu::resetTMarks(void) {	// resets marks in tstring
	tstart=0; tend=0;
	if (!fstart || !fend) {
		menuBar->setItemEnabled(COPY_ITEM, FALSE);		// disables "copy"
		menuBar->setItemEnabled(CUT_ITEM, FALSE);		// disables "cut"
	}
	viewChanged();
}

void dnaMenu::resetFMarks(void) {	// resets marks in fstring
	fstart=0; fend=0;
	if (!tstart || !tend) {
		menuBar->setItemEnabled(COPY_ITEM, FALSE);		// disables "copy"
		menuBar->setItemEnabled(CUT_ITEM, FALSE);		// disables "cut"
	}
	viewChanged();
}

void dnaMenu::settStart(void) {		// sets tstart to current position
	tstart=pos;
	if (tstart>dna.length()) tstart=dna.length();	// pos was too far right
	if (tend) {
		menuBar->setItemEnabled(COPY_ITEM, TRUE);		// enables "copy"
		menuBar->setItemEnabled(CUT_ITEM, TRUE);		// enables "cut"
	}
	viewChanged();
}

void dnaMenu::settEnd(void) {		// sets tend to current position
	tend=pos;
	if (tend>dna.length()) tend=dna.length();		// pos was too far right
	if (tstart) {
		menuBar->setItemEnabled(COPY_ITEM, TRUE);		// enables "copy"
		menuBar->setItemEnabled(CUT_ITEM, TRUE);		// enables "cut"
	}
	viewChanged();
}

void dnaMenu::setfStart(void) {		// sets fstart to current position
	fstart=pos;
	if (fstart>dna.length()) fstart=dna.length();	// pos was too far right
	if (fend) {
		menuBar->setItemEnabled(COPY_ITEM, TRUE);		// enables "copy"
		menuBar->setItemEnabled(CUT_ITEM, TRUE);		// enables "cut"
	}
	viewChanged();
}

void dnaMenu::setfEnd(void) {		// sets fend to current position
	fend=pos;
	if (fend>dna.length()) fend=dna.length();		// pos was too far right
	if (fstart) {
		menuBar->setItemEnabled(COPY_ITEM, TRUE);		// enables "copy"
		menuBar->setItemEnabled(CUT_ITEM, TRUE);		// enables "cut"
	}
	viewChanged();
}

void dnaMenu::markAll(void) {			// marks the whole DNA string
	fstart=1;
	tstart=1;
	fend=dna.length();
	tend=dna.length();
	menuBar->setItemEnabled(COPY_ITEM, TRUE);		// enables "copy"
	menuBar->setItemEnabled(CUT_ITEM, TRUE);		// enables "cut"
	viewChanged();
}

void dnaMenu::updateViewGL(void) {				// repaints the DNA graphic window

	GLfloat sx=glDnaBox->getScaleX();		// gets the scale for x-axis
	GLfloat sy=glDnaBox->getScaleY();		// gets the scale for y-axis
	GLfloat tx=-1+sx*0.4;					// calculates the translation in x-direction
	GLfloat ty=1-sy;						// calculates the translation in y-direction
	/* with this tx and ty the graphic always starts at the (circa) same left top corner of the window */

	GLfloat dist=glDnaBox->getDist();			// gets distance between two bases

	dnastring fstring(dna.getf()), tstring(dna.gett());
	uint n=pos, i, flen, tlen;
	uint w=((glDnaBox->width()-20)/(glDnaBox->getScale()))/57;	// numbers of bases that could shown fully in the window
												// critical implizit type cast

	// printf("pos: %ld n: %ld w: %ld\n",pos,n,w);

	uint tempfs=fstart, tempfe=fend, tempts=tstart, tempte=tend;		// temporary starts and ends for selections
	if (tempfs && tempfe && tempfs>tempfe) { i=tempfs; tempfs=tempfe; tempfe=i; }	// switches temporary starts and end if necessary
	if (tempts && tempte && tempts>tempte) { i=tempts; tempts=tempte; tempte=i; }

	if (end<=w || 2*n<w) n=1;			// whole string can be shown in window or the position n is smaller than w/2
	else if (2*n+w>2*dna.length()) {		// string is longer than the screen and current position n is bigger than end-w/2
		w--;
		n=end-(w/2)*2-1;		// calculates the right start n for cutting
		w--;				// display only w-2 bases at the end
	}
	else {
		n=n-(w-1)/2;		// position n is in the middle of a string, n is choosen s.t. it's always at the same position on screen
		if (n+w>=end) w=end-n-1;	// if it wants to show more bases than the DNA string has, it shows the most possible
	}

	// printf("pos: %ld n: %ld w: %ld\n",pos,n,w);

	bool shift=FALSE;		// is TRUE if DNA needs to be shifted, s.t. there is no broken yellow band at the left side
	if (n>1) {
		n--;			// gets one more base
		w++;			// and get one more at the end
		shift=TRUE;		// DNA on screen needs to be shifted
	}

	// printf("pos: %ld n: %ld w: %ld\n",pos,n,w);

	fstring.cut(n,n+w);	// cuts the part out that is only visible on the screen
	tstring.cut(n,n+w);
	flen=fstring.length();
	tlen=tstring.length();

	uint tempPos=pos-n;	// position on the screen

	// printf("%ld %ld %ld %ld \n",pos, n, tempPos, w);

	glLoadIdentity();					// for 5'-3' string
	glDnaBox->glTranslate(tx,ty);		// translates the scene in the top right corner...
	glDnaBox->glScale(sx,sy);			// ... and scales it right ...
	if (shift) glDnaBox->glTranslate(-dist);	// ... and shifts it one base to the left if necessary

	for (i=1;i<=flen;i++) {				// adds normal, marked or current bases
		switch (fstring.getc(i)) {
		  case 'A':	if (tempPos+1==i) glDnaBox->addCurrentA();
		  		else if (tempfs<=n+i-1 && n+i-1<=tempfe) glDnaBox->addMarkedA();
				else glDnaBox->addA();
				break;
		  case 'C':	if (tempPos+1==i) glDnaBox->addCurrentC();
		  		else if (tempfs<=n+i-1 && n+i-1<=tempfe) glDnaBox->addMarkedC();
		  		else glDnaBox->addC();
				break;
		  case 'G':	if (tempPos+1==i) glDnaBox->addCurrentG();
		  		else if (tempfs<=n+i-1 && n+i-1<=tempfe) glDnaBox->addMarkedG();
		  		else glDnaBox->addG();
				break;
		  case 'T':	if (tempPos+1==i) glDnaBox->addCurrentT();
		  		else if (tempfs<=n+i-1 && n+i-1<=tempfe) glDnaBox->addMarkedT();
		  		else glDnaBox->addT();
				break;
		  default :	break;
		}
		glDnaBox->glTranslate(dist);		// shifts the scene one base to the right
	}
	if (pos==end) glDnaBox->addEmpty();

	glLoadIdentity();					// for 3'-5' string
	glDnaBox->glTranslate(tx,ty);		// translates the scene in the top right corner...
	glDnaBox->glScale(sx,sy);			// ... and scales it right ...
	glDnaBox->glRotateHalf();			// ... and rotates the whole scene 180 degree around the z-axis ...
	if (shift) glDnaBox->glTranslate(dist);	// ... and shifts it one base to the left (here: right after rotation) if necessary

	for (i=1;i<=tlen;i++) {				// adds normal, marked or current bases
		switch (tstring.getc(i)) {
		  case 'A':	if (tempPos+1==i) glDnaBox->addCurrentA(TRUE);
		  		else if (tempts<=n+i-1 && n+i-1<=tempte) glDnaBox->addMarkedA(TRUE);
		  		else glDnaBox->addA(TRUE);
				break;
		  case 'C':	if (tempPos+1==i) glDnaBox->addCurrentC(TRUE);
		  		else if (tempts<=n+i-1 && n+i-1<=tempte) glDnaBox->addMarkedC(TRUE);
		  		else glDnaBox->addC(TRUE);
				break;
		  case 'G':	if (tempPos+1==i) glDnaBox->addCurrentG(TRUE);
		  		else if (tempts<=n+i-1 && n+i-1<=tempte) glDnaBox->addMarkedG(TRUE);
		  		else glDnaBox->addG(TRUE);
				break;
		  case 'T':	if (tempPos+1==i) glDnaBox->addCurrentT(TRUE);
		  		else if (tempts<=n+i-1 && n+i-1<=tempte) glDnaBox->addMarkedT(TRUE);
		  		else glDnaBox->addT(TRUE);
				break;
		  default :	break;
		}
		glDnaBox->glTranslate(-dist);	// shifts the scene one base to the right
	}
	if (pos==end) glDnaBox->addEmpty();

	sliderFocus();			// sets focus back to the slider
}

void dnaMenu::updateView(void) {		// repaints the DNA text window
	uint n=pos;
	uint w=(width()-12)/(dnaBox->fontMetrics()).width(' ')-2;	// numbers of bases that could shown fully in the window

	dnastring fstring(dna.getf()), tstring(dna.gett());
	uint i, tempPos;

	uint tempfs=fstart, tempfe=fend, tempts=tstart, tempte=tend;		// temporary starts and ends for selections
	if (tempfs && tempfe && tempfs>tempfe) { i=tempfs; tempfs=tempfe; tempfe=i; }	// switches temporary starts and end if necessary
	if (tempts && tempte && tempts>tempte) { i=tempts; tempts=tempte; tempte=i; }

	if (end<=w || 2*n<=w) {				// whole string can be shown in window or the position n is smaller than w/2
		n=1;
	}
	else if (2*n+w>=2*dna.length()) n=end-1-w;	// string is longer than the screen and current position n is bigger than end-w/2
	else n=n-w/2;		// position n is in the middle of a string, n is choosen s.t. it's always at the same position on screen

	fstring.cut(n,n+w);	// cuts the part out that is only visible on the screen
	tstring.cut(n,n+w);

	dnaBox->clear();		// clears the DNA text box
	tempPos=pos-n;		// position on the screen

	// printf("|%s|\n|%s|\n",fstring.data(),tstring.data());
	// printf("%ld %ld %ld %ld %ld %ld %ld %ld \n", pos, n, w, end, tempfs, tempfe,tempts,tempte);

	if (!tempfs || !tempfe || n+w<tempfs || n>tempfe) {	// selection in fstring is not visible on screen
		if (fstring.isEmpty()) dnaBox->insert(" ");
		else dnaBox->insert(fstring);
	}
	else {		// some selection is visible on the screen
		if (tempfs>n) {		// selected area starts in the "middle" of the screen
			if (tempfe<=MIN(n+w,end-1)) {		// selected area ends in the "middle" of the screen
				dnaBox->insert(fstring.left(tempfs-n));
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(fstring.mid(tempfs-n,tempfe-tempfs+1));
				dnaBox->setUnderline(FALSE);
				dnaBox->insert(fstring.right(MIN(w,end-2)-tempfe+n));
			}
			else {						// selected area end right outside the screen
				dnaBox->insert(fstring.left(tempfs-n));
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(fstring.right(w+1-tempfs+n));
				dnaBox->setUnderline(FALSE);
			}
		}
		else {			// selected area starts left outside the screen
			if (tempfe<MIN(n+w,end-1)) {		// selected area ends in the "middle" of the screen
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(fstring.left(tempfe-n+1));
				dnaBox->setUnderline(FALSE);
				dnaBox->insert(fstring.right(MIN(w,end-2)-tempfe+n));
			}
			else {						// selected area end right outside the screen
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(fstring);
				dnaBox->setUnderline(FALSE);
			}
		}
	}
	dnaBox->insert("\n");

	if (!tempts || !tempte || n+w<tempts || n>tempte) {	// same as above now for tstring
		if (tstring.isEmpty()) dnaBox->insert(" ");
		else dnaBox->insert(tstring);
	}
	else {
		if (tempts>n) {
			if (tempte<=MIN(n+w,end-1)) {
				dnaBox->insert(tstring.left(tempts-n));
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(tstring.mid(tempts-n,tempte-tempts+1));
				dnaBox->setUnderline(FALSE);
				dnaBox->insert(tstring.right(MIN(w,end-2)-tempte+n));
			}
			else {	// tend>n+w
				dnaBox->insert(tstring.left(tempts-n));
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(tstring.right(w+1-tempts+n));
				dnaBox->setUnderline(FALSE);
			}
		}
		else {	// tstart<=n
			if (tempte<MIN(n+w,end-1)) {
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(tstring.left(tempte-n+1));
				dnaBox->setUnderline(FALSE);
				dnaBox->insert(tstring.right(MIN(w,end-2)-tempte+n));
			}
			else {	// tend>=n+w
				dnaBox->setUnderline(TRUE);
				dnaBox->insert(tstring);
				dnaBox->setUnderline(FALSE);
			}
		}
	}

	QCString tempStr("");
	tempStr.insert(tempPos,'^');
	dnaBox->append(tempStr);	// adds a mark to the current position to show where the user is
	sliderFocus();			// sets focus back to the slider
}

void dnaMenu::updateZoom(float z) {		// repaints the zoom factor
	zoom=z;
	glDnaBox->setScale(zoom);		// sets scale factor in the OpenGL window
	QString temp;
	temp=temp.setNum(zoom*100)+"%";
	zoomLabel->setText(temp);
	zoomChanged();
}

void dnaMenu::zoomIn() {		// zooms in
	menuBar->setItemEnabled(ZOOM_OUT_ITEM, TRUE);		// enables zoom out
	float zNew, zOld=(dnaBox->fontMetrics()).width(' ')*1.0;
	if (zOld<zoomStd*3) {		// can not zoom in more
		int max=0;
		do {
			dnaBox->zoomIn();
			zNew=(dnaBox->fontMetrics()).width(' ')*1.0;
			if (zNew>=zoomStd*3) menuBar->setItemEnabled(ZOOM_IN_ITEM, FALSE);		// disables zoom in
			max++;
		} while (zOld==zNew && max<=3);		// some fonts could not zoom in, this would lead to and infinite loop
	}
	else zNew=zOld;
	updateZoom(zNew/zoomStd);		// updates zoom relatively to zoomStd
}

void dnaMenu::zoomOut() {
	menuBar->setItemEnabled(ZOOM_IN_ITEM, TRUE);		// enables zoom in
	float zNew, zOld=(dnaBox->fontMetrics()).width(' ')*1.0;
	if (zOld>zoomStd/2) {		// can not zoom out more
		int max=0;
		do {
			dnaBox->zoomOut();
			zNew=(dnaBox->fontMetrics()).width(' ')*1.0;
			if (zNew<=zoomStd/2) menuBar->setItemEnabled(ZOOM_OUT_ITEM, FALSE);		// disables zoom out
			max ++;
		} while (zOld==zNew && max<=3);		// some fonts could not zoom in, this would lead to and infinite loop
	}
	else zNew=zOld;
	updateZoom(zNew/zoomStd);		// updates zoom relatively to zoomStd
}

void dnaMenu::updateChange(bool change, bool reset) {
								// DNA string has changed and the marks have to be reset if reset is TRUE
	if (reset) resetMarks();		// resets all marks
	changed=change;		// set changed or not
	uint i=filename.findRev(QRegExp("[\\/]"));
	if (changed) {						// set new caption + "changed"
		setCaption(version + " - "+ filename.right(filename.length()-i-1)+" - changed");
	}
	else setCaption(version + " - "+ filename.right(filename.length()-i-1));
}

void dnaMenu::toggleText(void) {		// toggles between text and graphical mode
	if (textMode) {
		textMode=FALSE;		// switch to graphical mode

		dnaBox->hide();			// hides the DNA text box
		disconnect(this, SIGNAL(zoomChanged()), this, SLOT(updateView()));	// disconnects all signals
		disconnect(this, SIGNAL(posChanged()), this, SLOT(updateView()));
		disconnect(this, SIGNAL(viewChanged()), this, SLOT(updateView()));

		glDnaBox->show();		// shows the DNA graphic box
		connect(this, SIGNAL(zoomChanged()), glDnaBox, SLOT(updateGL()));	// connects all signals
		connect(this, SIGNAL(posChanged()), glDnaBox, SLOT(updateGL()));
		connect(this, SIGNAL(viewChanged()), glDnaBox, SLOT(updateGL()));

		zoomOut();
		zoomOut();
		zoomOut();
	}
	else {
		textMode=TRUE;			// switch to text mode

		glDnaBox->hide();		// hides the DNA graphic box
		disconnect(this, SIGNAL(zoomChanged()), glDnaBox, SLOT(updateGL()));	// disconnects all signals
		disconnect(this, SIGNAL(posChanged()), glDnaBox, SLOT(updateGL()));
		disconnect(this, SIGNAL(viewChanged()), glDnaBox, SLOT(updateGL()));

		dnaBox->show();			// shows the DNA text box
		connect(this, SIGNAL(zoomChanged()), this, SLOT(updateView()));		// connects all signals
		connect(this, SIGNAL(posChanged()), this, SLOT(updateView()));
		connect(this, SIGNAL(viewChanged()), this, SLOT(updateView()));

		zoomIn();
		zoomIn();
		zoomIn();
	}
	updateZoom(zoom);
}

void dnaMenu::findAgain(void) {			// finds the search argument again
	find(FROM_NOTHING,TRUE);
}

void dnaMenu::find(int from, bool again) {
	int res;
	if (dna.isEmpty()==BOTH_EMPTY) {		// DNA string is really empty
		QMessageBox messageBox;
		messageBox.information(0,
			version + " - Search DNA",
			"There is no DNA you can search in.",
			"&Ok", QString::null, QString::null, 0, 0);
		return;
	}
	if (!again) {						// starts new search
		if (from==FROM_NOTHING) {	// really new search
			findMenu *m=new findMenu(this, QString::null, TRUE);
			Q_CHECK_PTR(m);
			m->setFixedSize(m->sizeHint());
			m->setCaption(version + " - Find DNA");
			res=m->exec();
		}
		else res=from;
		switch (res) {
		   case FROM_CLIPBOARD: 	searchDNA=dnaClipBoard; break;		// searches DNA in clipboard
		   case FROM_FILE:		 	if (!openFind()) return; break;		// opens new DNA and searches for it
		   case FROM_CREATE: 		if (!createFind()) return; break;		// user can create new DNA and search for it
		   case FROM_SELECTION: 				// search for current selection
		   			QApplication::setOverrideCursor(waitCursor);
					searchDNA=dna;
					if (searchDNA.cut(fstart,fend,tstart,tend)) searchDNA.set("","");
					QApplication::restoreOverrideCursor();
					break;
		   default: 	return; break;
		}
	}
	int k=searchDNA.isEmpty();		// checks if search DNA is empty or not
	if (k==BOTH_EMPTY) {		// search DNA is really empty
		QMessageBox messageBox;
		messageBox.information(0,
			version + " - Search DNA",
			"There is no search argument, because the Search-DNA is empty.",
			"&Ok", QString::null, QString::null, 0, 0);
		return;
	}
	uint found;
	if (again) found=dna.find(searchDNA,pos+1);		// search again starts at pos+1
	else found=dna.find(searchDNA,pos);			// normal search starts at pos
	if (found) {								// search DNA was found at right side of pos
		switch (k) {
		     case NONE_EMPTY:	if (searchDNA.hangf()>0) {			// both single strings in search DNA are not empty
								fstart=found;
								fend=fstart+searchDNA.rlengthf()-1;
								tstart=found+searchDNA.hangf();
								tend=tstart+searchDNA.rlengtht()-1;
							}
							else {
								fstart=found-searchDNA.hangf();
								fend=fstart+searchDNA.rlengthf()-1;
								tstart=found;
								tend=tstart+searchDNA.rlengtht()-1;
							}
							break;
		     case FIVE_EMPTY:	tstart=found;					// 5'-3' string in search DNA is empty
							tend=tstart+searchDNA.rlengtht()-1;
							break;
		     case THREE_EMPTY:	fstart=found;					// 3'-5' string in search DNA is empty
							fend=fstart+searchDNA.rlengthf()-1;
							break;
		   			default:	break;
		}
		changePos(found);			// sets pos to found position
		QString temp;
		temp=temp.setNum(found);
		statusBar->message("Search string found at position "+temp+".",DISP_TIME);
	}
	else {						// search DNA was not found right side of pos
		QMessageBox messageBox;
		res=messageBox.information(0,
			version + " - Search DNA",
			"The search string was not found.\nShall I start searching at the beginning?",
			"&Ok", "&Cancel", QString::null, 0, 1);
		if (res==0) {
			found=dna.find(searchDNA);			// starts searching from the beginning
			if (found) {						// search DNA was found at right side of pos
				switch (k) {
		   		     case NONE_EMPTY:	if (searchDNA.hangf()>0) {
										fstart=found;
										fend=fstart+searchDNA.rlengthf()-1;
										tstart=found+searchDNA.hangf();
										tend=tstart+searchDNA.rlengtht()-1;
									}
									else {
										fstart=found-searchDNA.hangf();
										fend=fstart+searchDNA.rlengthf()-1;
										tstart=found;
										tend=tstart+searchDNA.rlengtht()-1;
									}
									break;
		   		     case FIVE_EMPTY:	tstart=found;
									tend=tstart+searchDNA.rlengtht()-1;
									break;
		   		     case THREE_EMPTY:	fstart=found;
									fend=fstart+searchDNA.rlengthf()-1;
									break;
		   		   			default:	break;
				}
				changePos(found);			// sets pos to found position
				QString temp;
				temp=temp.setNum(found);
				statusBar->message("Search string found at position "+temp+".",DISP_TIME);
			}
			else statusBar->message("Search string not found.",DISP_TIME);
		}
	}
}

void dnaMenu::findClip(void) {		// searches for the DNA in the clipboard
	find (FROM_CLIPBOARD);
}

void dnaMenu::findFile(void) {		// opens a file and search fo this DNA
	find (FROM_FILE);
}

void dnaMenu::findNew(void) {		// finds a new created search DNA
	find (FROM_CREATE);
}

void dnaMenu::findSel(void) {		// finds the current selection again
	find (FROM_SELECTION);
}

int dnaMenu::createFind(void) {				// shows a new DNA dialog to create a new DNA as search argument
	createMenu *m=new createMenu(this);
	Q_CHECK_PTR(m);
	m->setMinimumSize(m->sizeHint());
	m->setCaption(version + " - Create Search-DNA");
	int res=m->exec();
	if (res==2) {		// only if result was "OK" the new created DNA is choosen as search DNA
		QApplication::setOverrideCursor(waitCursor);
		searchDNA=m->getdna();
		QApplication::restoreOverrideCursor();
	}
	delete m;
	return(res);
}

int dnaMenu::openFind(void) {		// opens a new DNA file as search argument
	QString fn = QFileDialog::getOpenFileName(QString::null, QString::null, this);	// opens a DNA file
	if (!fn.isEmpty()) {
		QApplication::setOverrideCursor(waitCursor);
		searchDNA.read(fn);		// save the file data in search DNA
		QApplication::restoreOverrideCursor();
		return(1);
	}
	else return(0);
}

/******************************************************************************************************************************/

createMenu::createMenu(QWidget * parent, const char * name, bool modal) : MainWindow (parent,name,modal) {	// standard constructor
	butOk=new QPushButton("&Ok",this);
	Q_CHECK_PTR(butOk);
	butOk->setDefault(TRUE);

	butCancel=new QPushButton("C&ancel",this);
	Q_CHECK_PTR(butCancel);

	QVBoxLayout *mainLayout=new QVBoxLayout(this, 1);
	Q_CHECK_PTR(mainLayout);

	mainLayout->addWidget(dnaBox);		// adds the text DNA box
	glDnaBox->hide();				// hides the graphical DNA box, s.t. only one box at a time is shown
	mainLayout->addWidget(glDnaBox);	// adds the graphical DNA box

	QHBoxLayout *createLayout=new QHBoxLayout(mainLayout);
	Q_CHECK_PTR(createLayout);
	createLayout->addWidget(fGroup);
	createLayout->addWidget(tGroup);
	createLayout->addWidget(aGroup);

	QGridLayout *sliderLayout=new QGridLayout(mainLayout, 2, 3);
	Q_CHECK_PTR(sliderLayout);
	sliderLayout->addMultiCellWidget(slider,0,0,0,2);
	sliderLayout->addWidget(sliderCurrent,1,0);
	sliderLayout->addWidget(insLabel,1,1);
	sliderLayout->addWidget(sliderEnd,1,2);

	QHBoxLayout *buttonLayout=new QHBoxLayout(mainLayout);
	Q_CHECK_PTR(buttonLayout);
	buttonLayout->addWidget(butOk);
	buttonLayout->addWidget(butCancel);

	/* connects  the right buttons to the right signals */
	connect(butOk, SIGNAL(clicked()), this, SLOT(retOk()));
	connect(butCancel, SIGNAL(clicked()), this, SLOT(retCancel()));

	// connect(glDnaBox, SIGNAL(callScene()), this, SLOT(updateViewGL()));		// update the OpenGL scene if the signal is given

	connect(this, SIGNAL(posChanged()), this, SLOT(updateView()));			// every change needs to update the text window
	connect(this, SIGNAL(viewChanged()), this, SLOT(updateView()));
	connect(this, SIGNAL(dnaChanged(bool, bool)), this, SLOT (updateView(bool, bool)));
}

createMenu::~createMenu() {		// destructor
	delete butOk;
	delete butCancel;
}

dnadstring createMenu::getdna(void) {		// returns the current DNA
	return(dna);
}

void createMenu::retOk(void) {			// returns if DNA should be used

	if (dna.isnotCorrect()) {			// checks again if DNA is correct
		QMessageBox messageBox;
	     	messageBox.information(0,
			version + " - Incorrect DNA",
			"The actual DNA string is not correct.\nPlease read the manual to achieve a correct string.",
			"&Ok", QString::null, QString::null, 0, 0);
	}
	else done(2);
}

void createMenu::retCancel(void) {		// returns if DNA is discard
	done(0);
}

void createMenu::updateView(bool, bool) {		// repaints the DNA text window
	uint n=pos;
	uint w=(width()-12)/(dnaBox->fontMetrics()).width(' ')-2;	// numbers of bases that could shown fully in the window

	uint tempPos;
	dnastring fstring(dna.getf()), tstring(dna.gett());

	if (end<=w || 2*n<=w) n=1;				// whole string can be shown in window or the position n is smaller than w/2
	else if (2*n+w>=2*dna.length()) n=end-1-w;	// string is longer than the screen and current position n is bigger than end-w/2
	else n=n-w/2;		// position n is in the middle of a string, n is choosen s.t. it's always at the same position on screen

	fstring.cut(n,n+w);	// cuts the part out that is only visible on the screen
	tstring.cut(n,n+w);

	dnaBox->clear();		// clears the DNA text box
	tempPos=pos-n;		// position on the screen

	if (fstring.isEmpty()) dnaBox->insert(" ");
	else dnaBox->insert(fstring);
	dnaBox->insert("\n");
	if (tstring.isEmpty()) dnaBox->insert(" ");
	else dnaBox->insert(tstring);

	QCString tempStr("");
	tempStr.insert(tempPos,'^');
	dnaBox->append(tempStr);	// adds a mark to the current position to show where the user is
	sliderFocus();			// sets focus back to the slider
}

/******************************************************************************************************************************/

findMenu::findMenu(QWidget *parent, const char *name, bool modal, bool paste) : QDialog (parent,name,modal) {
	butOk=new QPushButton("&Ok",this);
	Q_CHECK_PTR(butOk);
	butOk->setDefault(TRUE);

	butCancel=new QPushButton("&Cancel",this);
	Q_CHECK_PTR(butCancel);

	if (paste) fGroup=new QButtonGroup(5, Qt::Vertical, "Concatenate from...",this);
	else fGroup=new QButtonGroup(5, Qt::Vertical, "Find from...",this);
	Q_CHECK_PTR(fGroup);
	fGroup->setExclusive(TRUE);
	fGroup->setRadioButtonExclusive(TRUE);

	QRadioButton *butDummy=new QRadioButton(fGroup);	// dummy buttons s.t. results are beginning with 1 and not 0
	Q_CHECK_PTR(butDummy);
	butDummy->setFixedHeight(1);
	butDummy->hide();

	butClip=new QRadioButton("Clip&board",fGroup);
	Q_CHECK_PTR(butClip);
	butClip->setChecked(TRUE);
	setResult(1);

	butFile=new QRadioButton("&File",fGroup);
	Q_CHECK_PTR(butFile);

	butSel=new QRadioButton("Current &Selection",fGroup);
	Q_CHECK_PTR(butSel);

	if (!paste) {
		butNew=new QRadioButton("&New DNA",fGroup);
		Q_CHECK_PTR(butNew);
	}

	findLayout=new QGridLayout(this, 2, 2, 1);
	Q_CHECK_PTR(findLayout);
	findLayout->addMultiCellWidget(fGroup,0,0,0,1);
	findLayout->addWidget(butOk,1,0);
	findLayout->addWidget(butCancel,1,1);

	/* connects  the right buttons to the right signals */
	connect(butOk, SIGNAL(clicked()), this, SLOT(retOk()));
	connect(butCancel, SIGNAL(clicked()), this, SLOT(retCancel()));
	connect(fGroup, SIGNAL(clicked(int)), this, SLOT(setResult(int)));
}

findMenu::~findMenu()  {		// destructor
	delete butOk;
	delete butCancel;
	delete fGroup;
	delete butClip;
	delete butSel;
	delete butNew;
	delete findLayout;
}

void findMenu::retOk(void) {			// returns and accepts the choice
	done(res);
}

void findMenu::retCancel(void) {		// returns if no choice is made
	done(0);
}

void findMenu::setResult(int id) {		// sets the result depending on the selection
	res=id;
}


