/* DNA-View - window.h
Here you find all definitions for the GUI */

/**
Copyright (C) 2003, 2007 Dominik Wagenfuehr <dominik.wagenfuehr@arcor.de>

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WINDOW_H
#define WINDOW_H

#include <qfiledialog.h>
#include <qdir.h>
#include <qapplication.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qslider.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qbuttongroup.h>
#include "dna.h"
#include "opengl.h"

#define DISP_TIME 5000			// time in ms to display messages in status bar

extern dnadstring dnaClipBoard;		// clipboard to copy a DNA string to
extern dnadstring searchDNA;		// DNA to search for
extern dnadstring undoDNA;		// last DNA before changes done for undo

class MainMenu : public QWidget {	// class for Main Menu
	Q_OBJECT

	QMenuBar *menuBar;

   public:
   	MainMenu(QWidget *parent=0, const char *name=0);		// standard constructor
	~MainMenu();									// destructor

   private slots:
	void newdna(bool open=FALSE);		// creates new window, if open is TRUE then it also opens a file
	void openin(void);				// opens file with DNA in new window
	void help(void);					// opens the help file in mozilla
	void about(void);					// shows an about message
	void aboutqt(void);				// shows an aboutQt message
};

class MainWindow : public QDialog {		// class for all DNA windows
	Q_OBJECT

	friend class dnaMenu;		// class dnaMenu declared as friend s.t. it can call private variables of MainWindow
	friend class createMenu;	// class createMenu declared as friend s.t. it can call private variables of MainWindow

	dnadstring dna;					// one dna double string for each window
	QTextEdit *dnaBox;				// box with the DNA in text mode
	glWindow *glDnaBox;				// box with the DNA in graphic mode
	QLabel *sliderCurrent, *sliderEnd;		// two labels with current position and last position
	QSlider *slider;				// slider to scroll left and right through DNA
	uint pos, end;				// current position pos and last position end
	QLabel *insLabel;			// label to show if insert or overwrite mode is set
	bool ins;					// insert or overwrite mode

	QButtonGroup *fGroup, *tGroup, *aGroup;		// groups for base buttons
	QPushButton *butA0, *butC0, *butG0, *butT0,		// different base buttons
		    *but0A, *but0C, *but0G, *but0T,
		    *butAT, *butCG, *butGC, *butTA;

	void insert(uint position, char fc, char tc);		// inserts at position the char fc in fstring and tc in tstring
	void del(uint position);						// deletes base pair at position

   protected:
	void resizeEvent(QResizeEvent *);		// event if user resizes the window
	void keyPressEvent(QKeyEvent *event);	// event if user press a key (needed for backspace, delete and insert)

   public:
   	MainWindow(QWidget *parent=0, const char *name=0,  bool modal=FALSE);		// standard constructor
	~MainWindow();													// destructor

   private slots:
	void updateSliderCurrent(void);	// repaints the current position label
	void updateSliderEnd(int n);		// repaints the end position label
	void updateInsert(void);		// repaints the insert label and change from insert to overwrite mode or vice versa
	void setSliderFocus(void);		// sets the focus to the slider, s.t. you can move it with the keyboard
	void changePos(int n);			// sets position new if slider has moved

	void insertA0(void);			// inserts a base pair
	void insertC0(void);
	void insertG0(void);
	void insertT0(void);
	void insert0A(void);
	void insert0C(void);
	void insert0G(void);
	void insert0T(void);
	void insertAT(void);
	void insertCG(void);
	void insertGC(void);
	void insertTA(void);

   signals:
	void posChanged(void);		// signal for changed position
	void endChanged(int n);		// signal for changed end
	void viewChanged(void);		// signal for changed view
	void sliderFocus(void);		// signal when slider needs focus
	void dnaChanged(bool change, bool reset);		// signal that DNA has changed, reset says if the marks has to be reset

};

class dnaMenu : public MainWindow {		// class for the normal DNA window
	Q_OBJECT

	QMenuBar *menuBar;
	QStatusBar *statusBar;
	QLabel *zoomLabel;			// label for zoom
	QString filename;				// filename to save the DNA
	float zoom, zoomStd;			// zoom factor and standard zoom factor
	uint fstart, fend, tstart, tend;		// positions for markings
	bool textMode;				// text mode on / off
	bool changed;				// DNA has changed (e.g. by cutting or blunting)

	int createFind(void);			// shows a new DNA dialog to create a new DNA as search argument
	int openFind(void);			// opens a new DNA file as search argument

   public:
   	dnaMenu(QWidget *parent=0, const char *name=0, bool modal=FALSE );		// standard constructor
	~dnaMenu();												// destructor
	QString getFileName(void) const;			// returns the current filename the DNA is saved under
	void opendna(void);					// opens a DNA file and reads the data

   private slots:
	void savedna(void);			// saves DNA with current filename
	void saveas(void);			// saves DNA with new filename
	void closedna(void);			// closes the window

	void resetMarks(void);			// resets all marks
	void resetFMarks(void);		// resets marks in fstring
	void resetTMarks(void);		// resets marks in tstring
	void settStart(void);			// sets tstart to current position
	void settEnd(void);			// sets tend to current position
	void setfStart(void);			// sets fstart to current position
	void setfEnd(void);			// sets tend to current position
	void markAll(void);			// marks the whole DNA string

	void setUndo(void);			// sets Undo Point
	void undo(void);			// restores last changes

	void cut(void);				// cuts a DNAstring at marks out
	void copy(void);			// copies a DNA string at marks to clipboard
	void blunt(void);			// blunts a DNA string
	void split(void);				// splits DNA double string in two single strings
	void stats(void);			// shows a statistic dialog

	void paste(int from=0);		// opens the paste dialog
	void pasteFile(void);			// opens a file and pastes this to the DNA
	void pasteClip(void);		// pastes the DNA in the clipboard
	void pasteSel(void);			// pastes the current selection

	void find(int from=0, bool again=FALSE);	// opens the find dialog or search again if again is TRUE
	void findAgain(void);					// finds the search argument again
	void findNew(void);				// finds a new created search DNA
	void findFile(void);			// opens a file and search fo this DNA
	void findClip(void);		// searches for the DNA in the clipboard
	void findSel(void);		// finds the current selection again

	void toggleText(void);		// toggles between text and graphical mode

	void zoomIn(void);			// zooms in
	void zoomOut(void);			// zooms out
	void updateView(void);		// repaints the DNA text window
	void updateViewGL(void);		// repaints the DNA graphic window
	void updateZoom(float z);		// repaints the zoom factor
	void updateChange(bool change, bool reset);		// DNA string has changed and the marks have to be reset if reset is TRUE

   signals:
	void zoomChanged(void);		// signal that the zoom has changed
};

class createMenu : public MainWindow {	// class for create DNA in find-routine
	Q_OBJECT

	QPushButton *butOk, *butCancel;		// different buttons

   public:
   	createMenu(QWidget * parent = 0, const char * name = 0, bool modal = FALSE);		// standard constructor
	~createMenu();														// destructor

	dnadstring getdna(void);		// returns the current DNA

   private slots:
	void retOk (void);			// returns if DNA should be used
	void retCancel(void);			// returns if DNA is discard
	void updateView(bool a=TRUE, bool b=TRUE);
		// repaints the DNA window; a and b are just for convenience, s.t. SIGNAL dnaChanged can connect to updateView
};

class findMenu : public QDialog {			// class for find dialog
	Q_OBJECT

	QGridLayout *findLayout;

	QPushButton *butOk, *butCancel;

	QRadioButton *butFile, *butClip, *butNew, *butSel;		// different buttons
	QButtonGroup *fGroup;						// button group

	int res;		// result which is returned

   public:
   	findMenu(QWidget * parent = 0, const char * name = 0, bool modal = FALSE, bool paste=FALSE); 		// standard constructor
	~findMenu();													// destructor

   private slots:
	void retOk (void);			// returns and accepts the choice
	void retCancel(void);			// returns if no choice is made
	void setResult(int id);			// sets the result depending on the selection

};

#endif
